# -*- coding: utf-8 -*-

"""Main module."""

import click

import datetime
import hashlib
import json
import logging
import setproctitle
import sys, os
import yaml

from asyncio import run, gather, wait
from porerefiner import models, samplesheets, jobs
from porerefiner.models import Job, Run, SampleSheet
import porerefiner.cli_utils as cli_utils
import porerefiner.jobs.submitters as submitters
from pathlib import Path

from porerefiner.rpc import start_server
from porerefiner.fsevents import start_fs_watchdog, start_run_end_polling, start_job_polling, in_progress_run_update

from porerefiner.daemon import daemon

log = logging.getLogger('porerefiner.service')

# Load plugins, if any

import pkg_resources

discovered_plugins = {
    entry_point.name: entry_point.load()
    for entry_point
    in pkg_resources.iter_entry_points('porerefiner.plugins')
}


async def serve(config_file, db_path=None, db_pragmas=None, wdog_settings=None, server_settings=None, system_settings=None):
    "Initialize and gather coroutines"
    if not all([db_path, db_pragmas, wdog_settings, server_settings, system_settings]): #need to defer loading config for testing purposes
        from porerefiner.config import Config
        config = Config(config_file).config
        db_path=config['database']['path']
        db_pragmas=config['database']['pragmas']
        wdog_settings=config['nanopore']
        server_settings=config['server']
        system_settings=config['porerefiner']
    models._db.init(db_path, db_pragmas)
    [cls.create_table(safe=True) for cls in models.REGISTRY]
    try:
        results = await gather(
                            start_server(**server_settings),
                            start_fs_watchdog(**wdog_settings),
                            start_run_end_polling(**system_settings),
                            start_job_polling(**system_settings),
                            in_progress_run_update()
                        )
    finally:
        log.warning("Shutting down...")

# bit of complexity here to handle different defaults for privileged vs normal users

path_if_root = Path('/etc/porerefiner/config.yaml')

default_config = lambda: os.environ.get('POREREFINER_CONFIG',
                                        lambda: (Path.home() / '.porerefiner' / 'config.yaml',
                                                path_if_root)[os.geteuid == 0 or path_if_root.exists()])

config = click.option('--config',
                      prompt=('path to config file', False)['POREREFINER_CONFIG' in os.environ],
                      default=default_config(),
                      show_envvar=True,
                      type=cli_utils.PathPath())


@click.group()
@click.option('-v', '--verbose', is_flag=True)
def cli(verbose):
    logging.basicConfig(stream=sys.stdout,
                        style='{',
                        format="{levelname} {module}:{message}",
                        level=(logging.DEBUG, logging.INFO)[verbose])

@cli.command()
@config
@click.option('--nanopore_dir')
@click.option('--client', '-c', 'client_only', is_flag=True)
def init(config, nanopore_dir='/var/lib/minknow/data', client_only=False):
    "Find the Nanopore output directory and create the config file."
    if config.exists():
        if click.confirm(f"delete existing config file at {config}?"):
            config.unlink()
    if click.confirm(f"create PoreRefiner config at {config}?"):
        sock_path = click.prompt(f"location of porerefiner RPC socket?", default=config.parent / 'porerefiner.sock', show_default=True)
        if not client_only:
            db_path = click.prompt(f"location of database?", default=config.parent / 'database.db', show_default=True)
            nan_path = click.prompt("nanopore data output location?", default=nanopore_dir, show_default=True)
        from porerefiner.config import Config
        Config.new_config_file(config, client_only=client_only, database_path=db_path, socket_path=sock_path)
        click.echo(f'''export POREREFINER_CONFIG="{config}"''')

@cli.command()
@config
@click.option('-d', '--daemonize', 'demonize', is_flag=True, default=False)
def start(config, demonize=False):
    "Start the PoreRefiner service."
    setproctitle.setproctitle("porerefiner")
    log = logging.getLogger('porerefiner')
    if demonize:
        log.info("Starting daemon...")
        with daemon.DaemonContext():
            run(serve(config))
    else:
        log.info("Starting server, ctl-C to stop.")
        run(serve(config))
    return 0


@cli.group()
def info():
    "Get info about configurable event handlers - notifiers, submitters, and jobs."
    pass

@info.command(name='notifiers')
@click.argument('notifier', default=None, required=False)
def _notifiers(notifier=None):
    "Notifiers that are installed and can be configured."
    from porerefiner.notifiers import REGISTRY, Notifier
    if not notifier:
        for name, notifier in REGISTRY.items():
            if notifier is not Notifier:
                click.echo(name)
    else:
        try:
            click.echo(REGISTRY[notifier].get_configurable_options())
        except KeyError:
            click.echo(f"Notifier '{notifier}'' not installed.", err=True)

@info.command(name='submitters')
@click.argument('submitter', default=None, required=False)
def _submitters(submitter=None):
    "Job submitters that are installed and can be configured."
    from porerefiner.jobs.submitters import REGISTRY, Submitter
    if not submitter:
        for name, submitter in REGISTRY.items():
            if submitter is not Submitter:
                click.echo(name)
    else:
        try:
            click.echo(REGISTRY[submitter].get_configurable_options())
        except KeyError:
            click.echo(f"Notifier '{submitter}'' not installed.", err=True)

@info.command(name='jobs')
@click.argument('job', default=None, required=False)
def _jobs(job=None):
    "Jobs that are installed and can be configured."
    from porerefiner.jobs import CLASS_REGISTRY, FileJob, RunJob, AbstractJob
    if not job:
        for name, job in CLASS_REGISTRY.items():
            if job is not FileJob and job is not RunJob and job is not AbstractJob:
                print(name)
    else:
        try:
            click.echo(CLASS_REGISTRY[job].get_configurable_options())
        except KeyError:
            click.echo(f"Notifier '{job}'' not installed.", err=True)

@cli.group()
def reset():
    "Utility function to reset various state."
    pass

@reset.command()
@click.argument('status', default="QUEUED", type=click.Choice([v for v, _ in Job.statuses], case_sensitive=True))
def _jobs(status):
    "Reset all jobs to a particular status."
    if click.confirm(f"This will set all jobs to {status} status. Are you sure?"):
        Job.update(status=status).execute()
        click.echo(f"Jobs set to {status}.")

@reset.command()
@click.argument('status', default="RUNNING", type=click.Choice([v for v, _ in Run.statuses], case_sensitive=True))
@click.option('--run', 'run_name', default=None)
@config
def runs(status, config, run_name=None):
    "Reset all runs to in-progress status."
    from porerefiner.config import Config
    config = Config(config).config
    db_path=config['database']['path']
    db_pragmas=config['database']['pragmas']
    models._db.init(db_path, db_pragmas)
    if run_name:
        if click.confirm("This will set run {run_name} to {status} status. Are you sure?"):
            Run.update(status=status).where(alt_name=run_name).execute()
            click.echo(f"Run {run_name} set to {status}.")
    else:
        if click.confirm(f"This will set all runs to {status} status, triggering notifiers and jobs in the next hour. Are you sure?"):
            Run.update(status=status).execute()
            click.echo(f'Runs set to {status}.')

@reset.command()
@config
def database(config):
    "Reset database to empty state."
    if click.confirm("This will delete the porerefiner database. Are you sure?"):
        import porerefiner.config
        config = porerefiner.config.Config(config).config
        db_path=config['database']['path']
        db_pragmas=config['database']['pragmas']
        if Path(db_path).exists():
            Path(db_path).unlink()
        models._db.init(db_path, db_pragmas)
        models._db.connect()
        models._db.create_tables(models.REGISTRY)


@reset.command(name='samplesheets')
def _samplesheets(): #TODO
    "Clear samplesheets that aren't attached to any run."
    click.echo("clear sheets")

@cli.group(name="list")
@config
def _list(config):
    "List job system stuff."
    from porerefiner.config import Config
    config = Config(config).config
    db_path=config['database']['path']
    db_pragmas=config['database']['pragmas']
    models._db.init(db_path, db_pragmas)

@_list.command(name='jobs')
def _jobs():
    "List the configurable, configured, and spawned jobs."
    import porerefiner.jobs
    click.echo("Installed job modules:")
    click.echo(yaml.dump(list(porerefiner.jobs.CLASS_REGISTRY.keys())))
    click.echo("Configured jobs:")
    click.echo(yaml.dump(porerefiner.jobs.JOBS))
    click.echo("Spawned jobs:")
    [click.echo(yaml.dump(job)) for job in Job.select().dicts()]

@_list.command(name='submitters')
def _submitters():
    "List the configureable and configured submitters."
    import porerefiner.jobs
    click.echo("Installed submitters:")
    click.echo(yaml.dump(list(porerefiner.jobs.submitters.REGISTRY.keys())))
    click.echo("Configured submitters:")
    click.echo(yaml.dump(porerefiner.jobs.submitters.SUBMITTERS))

@_list.command()
def notifiers():
    "List the configurable and configured notifiers."
    import porerefiner.notifiers
    click.echo("Installed notifiers:")
    click.echo(yaml.dump(list(porerefiner.notifiers.REGISTRY.keys())))
    click.echo("Configured notifiers:")
    click.echo(yaml.dump(porerefiner.notifiers.NOTIFIERS))

@cli.group()
@config
def verify(config):
    "Run various checks."
    from porerefiner.config import Config
    config = Config(config).config

@verify.command(name='jobs')
@click.argument('sample_sheet', type=cli_utils.PathPath())
@click.argument('data_file', type=cli_utils.PathPath())
def _jobs(sample_sheet, data_file):
    "Verify job configuration by actually running them on sample data, through their submitter."
    # create an in-mem database
    from peewee import SqliteDatabase
    db = SqliteDatabase(":memory:", pragmas={'foreign_keys':1}, autoconnect=False)
    db.bind(models.REGISTRY, bind_refs=True, bind_backrefs=True)
    db.connect()
    db.create_tables(models.REGISTRY)

    # create a run
    ru = Run.create(name=data_file.parent.name,
                     ended=datetime.datetime.now(),
                     status='DONE',
                     path=data_file.parent)

    # load the sample sheet
    # save the sample to the run
    with open(sample_sheet, 'rb') as sheet:
        ss = SampleSheet.new_sheet_from_message(
            sheet=(samplesheets.load_from_csv,
                   samplesheets.load_from_excel)['xslx' in sample_sheet.name](sheet),
            run=ru
        )

    with open(data_file, 'rb') as data:
        hash = hashlib.md5()
        hash.update(data.read())
    md5 = hash.hexdigest()

    # create a file and add it to the run
    fi = models.File.create(path=data_file,
                            run=ru,
                            checksum=md5,
                            )

    async def fileJob(job):
        j = fi.spawn(job)
        await jobs.submit_job(j)
        while j.status not in ('DONE', 'FAILED'):
            await jobs.poll_active_job(j)

    async def runJob(job):
        j = ru.spawn(job)
        await jobs.submit_job(j)
        while j.status not in ('DONE', 'FAILED'):
            await jobs.poll_active_job(j)

    async def task():
        await gather(*[fileJob(job) for job in jobs.JOBS.FILES] +
                      [runJob(job) for job in jobs.JOBS.RUNS])

    run(task())

@verify.command(name='submitters')
def _submitters():
    "Verify configuration of job submitters by running their tests."
    async def test_all():
        for submitter in submitters.SUBMITTERS:
            click.echo(f'Running {type(submitter).__name__} integration no-op test')
            await submitter.test_noop()
    run(test_all())

@verify.command(name='notifiers')
def _notifiers():
    "Verify notifiers by sending notifications."
    from porerefiner.notifiers import NOTIFIERS
    async def test_all():
        for notifier in NOTIFIERS:
            click.echo(f"Running {type(notifier).__name__}")
            await notifier.notify(None)
    run(test_all())




if __name__ == '__main__':
    sys.exit(cli())                   # pragma: no cover



from tests import paths

from unittest import TestCase, skip
from unittest.mock import patch

from porerefiner import cli

class TestCli(TestCase):

    @skip("not sure how to test these")
    def test_ps(self, server=False, config=False, output_format=False, extend=False, _all=False, tags=[]):
        cli.ps(output_format, extend)
        assert False



from porerefiner.app import app
from porerefiner.cli_utils import server
from porerefiner.samplesheets import load_from_csv
from porerefiner.protocols.porerefiner.rpc.porerefiner_pb2 import RunRequest, RunListRequest, RunAttachRequest, RunRsyncRequest

from asyncio import run
from flask import render_template, current_app
import subprocess

from os import environ

async def list_run_runner():
    config = current_app.config['config_file']
    with server(config) as serv:
        return await serv.GetRuns(RunListRequest(all=True))

def list_runs():
    return run(list_run_runner()).runs.runs

@app.route('/attach')
@app.route('/api/form/attach')
def form():
    return render_template('submit.html', runs=list_runs(), hostname=current_app.config['host'])

@app.route('/')
@app.route('/view')
def runs():
    return render_template('view.html', runs=list_runs(), hostname=current_app.config['host'])

@app.route('/view/<int:run_id>')
def view_run(run_id):
    async def get_run(run_id):
        config = current_app.config['config_file']
        with server(config) as serv:
            return await serv.GetRunInfo(RunRequest(id=run_id))
    return render_template('run_view.html', run=run(get_run(run_id)).run)

@app.route('/template')
def template():
    return """porerefiner_ver,1.0.0
pool_id,
sequencing_kit,
sample_id,accession,barcode_id,organism,extraction_kit,comment,user
""", 200, {'Content-Type':'text/csv'}



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: minknow/rpc/instance.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import acquisition_pb2 as minknow_dot_rpc_dot_acquisition__pb2
from . import device_pb2 as minknow_dot_rpc_dot_device__pb2
from . import protocol_pb2 as minknow_dot_rpc_dot_protocol__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='minknow/rpc/instance.proto',
  package='ont.rpc.instance',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x1aminknow/rpc/instance.proto\x12\x10ont.rpc.instance\x1a\x1dminknow/rpc/acquisition.proto\x1a\x18minknow/rpc/device.proto\x1a\x1aminknow/rpc/protocol.proto\"\x17\n\x15GetVersionInfoRequest\"\x85\x03\n\x16GetVersionInfoResponse\x12H\n\x07minknow\x18\x01 \x01(\x0b\x32\x37.ont.rpc.instance.GetVersionInfoResponse.MinknowVersion\x12\x11\n\tprotocols\x18\x02 \x01(\t\x12\x1c\n\x14\x64istribution_version\x18\x03 \x01(\t\x12X\n\x13\x64istribution_status\x18\x04 \x01(\x0e\x32;.ont.rpc.instance.GetVersionInfoResponse.DistributionStatus\x1aK\n\x0eMinknowVersion\x12\r\n\x05major\x18\x01 \x01(\x05\x12\r\n\x05minor\x18\x02 \x01(\x05\x12\r\n\x05patch\x18\x03 \x01(\x05\x12\x0c\n\x04\x66ull\x18\x04 \x01(\t\"I\n\x12\x44istributionStatus\x12\x0b\n\x07UNKNOWN\x10\x00\x12\n\n\x06STABLE\x10\x01\x12\x0c\n\x08UNSTABLE\x10\x02\x12\x0c\n\x08MODIFIED\x10\x03\"\x1d\n\x1bGetOutputDirectoriesRequest\"?\n\x11OutputDirectories\x12\x0e\n\x06output\x18\x01 \x01(\t\x12\x0b\n\x03log\x18\x02 \x01(\t\x12\r\n\x05reads\x18\x03 \x01(\t\"$\n\"GetDefaultOutputDirectoriesRequest\")\n\x19SetOutputDirectoryRequest\x12\x0c\n\x04path\x18\x01 \x01(\t\"\x1c\n\x1aSetOutputDirectoryResponse\"(\n\x18SetReadsDirectoryRequest\x12\x0c\n\x04path\x18\x01 \x01(\t\"\x1b\n\x19SetReadsDirectoryResponse\"\xe0\x01\n\x17\x46ilesystemDiskSpaceInfo\x12\x15\n\rfilesystem_id\x18\x01 \x01(\t\x12\x17\n\x0f\x62ytes_available\x18\x02 \x01(\x04\x12\x16\n\x0e\x62ytes_capacity\x18\x03 \x01(\x04\x12\x0c\n\x04what\x18\x04 \x03(\t\x12\x1d\n\x15\x62ytes_to_stop_cleanly\x18\x05 \x01(\x04\x12\x1f\n\x17\x62ytes_when_alert_issued\x18\x06 \x01(\x04\x12\x17\n\x0frecommend_alert\x18\x07 \x01(\x08\x12\x16\n\x0erecommend_stop\x18\x08 \x01(\x08\"\x19\n\x17GetDiskSpaceInfoRequest\"i\n\x18GetDiskSpaceInfoResponse\x12M\n\x1a\x66ilesystem_disk_space_info\x18\x01 \x03(\x0b\x32).ont.rpc.instance.FilesystemDiskSpaceInfo\"\x15\n\x13GetMachineIdRequest\"*\n\x14GetMachineIdResponse\x12\x12\n\nmachine_id\x18\x01 \x01(\t\"\x14\n\x12GetHostTypeRequest\"\xc0\x01\n\x13GetHostTypeResponse\x12\x41\n\thost_type\x18\x01 \x01(\x0e\x32..ont.rpc.instance.GetHostTypeResponse.HostType\"f\n\x08HostType\x12\x06\n\x02PC\x10\x00\x12\x14\n\x10PROMETHION_ALPHA\x10\x01\x12\x0b\n\x07GRIDION\x10\x02\x12\x13\n\x0fPROMETHION_BETA\x10\x03\x12\t\n\x05MINIT\x10\x04\x12\x0f\n\x0bMINION_MK1C\x10\x05\"\x1f\n\x1dStreamInstanceActivityRequest\"\x92\x01\n\nDeviceInfo\x12H\n\x0c\x64\x65vice_state\x18\x01 \x01(\x0e\x32\x32.ont.rpc.device.GetDeviceStateResponse.DeviceState\x12:\n\x0b\x64\x65vice_info\x18\x02 \x01(\x0b\x32%.ont.rpc.device.GetDeviceInfoResponse\"\xa9\x05\n\x1eStreamInstanceActivityResponse\x12\x33\n\x0b\x64\x65vice_info\x18\x01 \x01(\x0b\x32\x1c.ont.rpc.instance.DeviceInfoH\x00\x12\x41\n\x0e\x66low_cell_info\x18\x02 \x01(\x0b\x32\'.ont.rpc.device.GetFlowCellInfoResponseH\x00\x12>\n\x11protocol_run_info\x18\x03 \x01(\x0b\x32!.ont.rpc.protocol.ProtocolRunInfoH\x00\x12G\n\x14\x61\x63quisition_run_info\x18\x04 \x01(\x0b\x32\'.ont.rpc.acquisition.AcquisitionRunInfoH\x00\x12[\n\x10\x66low_cell_health\x18\x05 \x01(\x0b\x32?.ont.rpc.instance.StreamInstanceActivityResponse.FlowCellHealthH\x00\x12\x45\n\ryield_summary\x18\x06 \x01(\x0b\x32,.ont.rpc.acquisition.AcquisitionYieldSummaryH\x00\x1a\xd1\x01\n\x0e\x46lowCellHealth\x12\x7f\n\x19\x63hannel_state_percentages\x18\x01 \x03(\x0b\x32\\.ont.rpc.instance.StreamInstanceActivityResponse.FlowCellHealth.ChannelStatePercentagesEntry\x1a>\n\x1c\x43hannelStatePercentagesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x02:\x02\x38\x01\x42\x0e\n\x0cstream_value2\x87\x08\n\x0fInstanceService\x12g\n\x10get_version_info\x12\'.ont.rpc.instance.GetVersionInfoRequest\x1a(.ont.rpc.instance.GetVersionInfoResponse\"\x00\x12n\n\x16get_output_directories\x12-.ont.rpc.instance.GetOutputDirectoriesRequest\x1a#.ont.rpc.instance.OutputDirectories\"\x00\x12}\n\x1eget_default_output_directories\x12\x34.ont.rpc.instance.GetDefaultOutputDirectoriesRequest\x1a#.ont.rpc.instance.OutputDirectories\"\x00\x12s\n\x14set_output_directory\x12+.ont.rpc.instance.SetOutputDirectoryRequest\x1a,.ont.rpc.instance.SetOutputDirectoryResponse\"\x00\x12p\n\x13set_reads_directory\x12*.ont.rpc.instance.SetReadsDirectoryRequest\x1a+.ont.rpc.instance.SetReadsDirectoryResponse\"\x00\x12n\n\x13get_disk_space_info\x12).ont.rpc.instance.GetDiskSpaceInfoRequest\x1a*.ont.rpc.instance.GetDiskSpaceInfoResponse\"\x00\x12\x61\n\x0eget_machine_id\x12%.ont.rpc.instance.GetMachineIdRequest\x1a&.ont.rpc.instance.GetMachineIdResponse\"\x00\x12^\n\rget_host_type\x12$.ont.rpc.instance.GetHostTypeRequest\x1a%.ont.rpc.instance.GetHostTypeResponse\"\x00\x12\x81\x01\n\x18stream_instance_activity\x12/.ont.rpc.instance.StreamInstanceActivityRequest\x1a\x30.ont.rpc.instance.StreamInstanceActivityResponse\"\x00\x30\x01\x62\x06proto3')
  ,
  dependencies=[minknow_dot_rpc_dot_acquisition__pb2.DESCRIPTOR,minknow_dot_rpc_dot_device__pb2.DESCRIPTOR,minknow_dot_rpc_dot_protocol__pb2.DESCRIPTOR,])



_GETVERSIONINFORESPONSE_DISTRIBUTIONSTATUS = _descriptor.EnumDescriptor(
  name='DistributionStatus',
  full_name='ont.rpc.instance.GetVersionInfoResponse.DistributionStatus',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='STABLE', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UNSTABLE', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='MODIFIED', index=3, number=3,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=475,
  serialized_end=548,
)
_sym_db.RegisterEnumDescriptor(_GETVERSIONINFORESPONSE_DISTRIBUTIONSTATUS)

_GETHOSTTYPERESPONSE_HOSTTYPE = _descriptor.EnumDescriptor(
  name='HostType',
  full_name='ont.rpc.instance.GetHostTypeResponse.HostType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='PC', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PROMETHION_ALPHA', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GRIDION', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PROMETHION_BETA', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='MINIT', index=4, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='MINION_MK1C', index=5, number=5,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1369,
  serialized_end=1471,
)
_sym_db.RegisterEnumDescriptor(_GETHOSTTYPERESPONSE_HOSTTYPE)


_GETVERSIONINFOREQUEST = _descriptor.Descriptor(
  name='GetVersionInfoRequest',
  full_name='ont.rpc.instance.GetVersionInfoRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=133,
  serialized_end=156,
)


_GETVERSIONINFORESPONSE_MINKNOWVERSION = _descriptor.Descriptor(
  name='MinknowVersion',
  full_name='ont.rpc.instance.GetVersionInfoResponse.MinknowVersion',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='major', full_name='ont.rpc.instance.GetVersionInfoResponse.MinknowVersion.major', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='minor', full_name='ont.rpc.instance.GetVersionInfoResponse.MinknowVersion.minor', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='patch', full_name='ont.rpc.instance.GetVersionInfoResponse.MinknowVersion.patch', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='full', full_name='ont.rpc.instance.GetVersionInfoResponse.MinknowVersion.full', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=398,
  serialized_end=473,
)

_GETVERSIONINFORESPONSE = _descriptor.Descriptor(
  name='GetVersionInfoResponse',
  full_name='ont.rpc.instance.GetVersionInfoResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='minknow', full_name='ont.rpc.instance.GetVersionInfoResponse.minknow', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='protocols', full_name='ont.rpc.instance.GetVersionInfoResponse.protocols', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='distribution_version', full_name='ont.rpc.instance.GetVersionInfoResponse.distribution_version', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='distribution_status', full_name='ont.rpc.instance.GetVersionInfoResponse.distribution_status', index=3,
      number=4, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_GETVERSIONINFORESPONSE_MINKNOWVERSION, ],
  enum_types=[
    _GETVERSIONINFORESPONSE_DISTRIBUTIONSTATUS,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=159,
  serialized_end=548,
)


_GETOUTPUTDIRECTORIESREQUEST = _descriptor.Descriptor(
  name='GetOutputDirectoriesRequest',
  full_name='ont.rpc.instance.GetOutputDirectoriesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=550,
  serialized_end=579,
)


_OUTPUTDIRECTORIES = _descriptor.Descriptor(
  name='OutputDirectories',
  full_name='ont.rpc.instance.OutputDirectories',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='output', full_name='ont.rpc.instance.OutputDirectories.output', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='log', full_name='ont.rpc.instance.OutputDirectories.log', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='reads', full_name='ont.rpc.instance.OutputDirectories.reads', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=581,
  serialized_end=644,
)


_GETDEFAULTOUTPUTDIRECTORIESREQUEST = _descriptor.Descriptor(
  name='GetDefaultOutputDirectoriesRequest',
  full_name='ont.rpc.instance.GetDefaultOutputDirectoriesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=646,
  serialized_end=682,
)


_SETOUTPUTDIRECTORYREQUEST = _descriptor.Descriptor(
  name='SetOutputDirectoryRequest',
  full_name='ont.rpc.instance.SetOutputDirectoryRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='path', full_name='ont.rpc.instance.SetOutputDirectoryRequest.path', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=684,
  serialized_end=725,
)


_SETOUTPUTDIRECTORYRESPONSE = _descriptor.Descriptor(
  name='SetOutputDirectoryResponse',
  full_name='ont.rpc.instance.SetOutputDirectoryResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=727,
  serialized_end=755,
)


_SETREADSDIRECTORYREQUEST = _descriptor.Descriptor(
  name='SetReadsDirectoryRequest',
  full_name='ont.rpc.instance.SetReadsDirectoryRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='path', full_name='ont.rpc.instance.SetReadsDirectoryRequest.path', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=757,
  serialized_end=797,
)


_SETREADSDIRECTORYRESPONSE = _descriptor.Descriptor(
  name='SetReadsDirectoryResponse',
  full_name='ont.rpc.instance.SetReadsDirectoryResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=799,
  serialized_end=826,
)


_FILESYSTEMDISKSPACEINFO = _descriptor.Descriptor(
  name='FilesystemDiskSpaceInfo',
  full_name='ont.rpc.instance.FilesystemDiskSpaceInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='filesystem_id', full_name='ont.rpc.instance.FilesystemDiskSpaceInfo.filesystem_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='bytes_available', full_name='ont.rpc.instance.FilesystemDiskSpaceInfo.bytes_available', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='bytes_capacity', full_name='ont.rpc.instance.FilesystemDiskSpaceInfo.bytes_capacity', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='what', full_name='ont.rpc.instance.FilesystemDiskSpaceInfo.what', index=3,
      number=4, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='bytes_to_stop_cleanly', full_name='ont.rpc.instance.FilesystemDiskSpaceInfo.bytes_to_stop_cleanly', index=4,
      number=5, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='bytes_when_alert_issued', full_name='ont.rpc.instance.FilesystemDiskSpaceInfo.bytes_when_alert_issued', index=5,
      number=6, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='recommend_alert', full_name='ont.rpc.instance.FilesystemDiskSpaceInfo.recommend_alert', index=6,
      number=7, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='recommend_stop', full_name='ont.rpc.instance.FilesystemDiskSpaceInfo.recommend_stop', index=7,
      number=8, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=829,
  serialized_end=1053,
)


_GETDISKSPACEINFOREQUEST = _descriptor.Descriptor(
  name='GetDiskSpaceInfoRequest',
  full_name='ont.rpc.instance.GetDiskSpaceInfoRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1055,
  serialized_end=1080,
)


_GETDISKSPACEINFORESPONSE = _descriptor.Descriptor(
  name='GetDiskSpaceInfoResponse',
  full_name='ont.rpc.instance.GetDiskSpaceInfoResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='filesystem_disk_space_info', full_name='ont.rpc.instance.GetDiskSpaceInfoResponse.filesystem_disk_space_info', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1082,
  serialized_end=1187,
)


_GETMACHINEIDREQUEST = _descriptor.Descriptor(
  name='GetMachineIdRequest',
  full_name='ont.rpc.instance.GetMachineIdRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1189,
  serialized_end=1210,
)


_GETMACHINEIDRESPONSE = _descriptor.Descriptor(
  name='GetMachineIdResponse',
  full_name='ont.rpc.instance.GetMachineIdResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='machine_id', full_name='ont.rpc.instance.GetMachineIdResponse.machine_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1212,
  serialized_end=1254,
)


_GETHOSTTYPEREQUEST = _descriptor.Descriptor(
  name='GetHostTypeRequest',
  full_name='ont.rpc.instance.GetHostTypeRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1256,
  serialized_end=1276,
)


_GETHOSTTYPERESPONSE = _descriptor.Descriptor(
  name='GetHostTypeResponse',
  full_name='ont.rpc.instance.GetHostTypeResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='host_type', full_name='ont.rpc.instance.GetHostTypeResponse.host_type', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _GETHOSTTYPERESPONSE_HOSTTYPE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1279,
  serialized_end=1471,
)


_STREAMINSTANCEACTIVITYREQUEST = _descriptor.Descriptor(
  name='StreamInstanceActivityRequest',
  full_name='ont.rpc.instance.StreamInstanceActivityRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1473,
  serialized_end=1504,
)


_DEVICEINFO = _descriptor.Descriptor(
  name='DeviceInfo',
  full_name='ont.rpc.instance.DeviceInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='device_state', full_name='ont.rpc.instance.DeviceInfo.device_state', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='device_info', full_name='ont.rpc.instance.DeviceInfo.device_info', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1507,
  serialized_end=1653,
)


_STREAMINSTANCEACTIVITYRESPONSE_FLOWCELLHEALTH_CHANNELSTATEPERCENTAGESENTRY = _descriptor.Descriptor(
  name='ChannelStatePercentagesEntry',
  full_name='ont.rpc.instance.StreamInstanceActivityResponse.FlowCellHealth.ChannelStatePercentagesEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='ont.rpc.instance.StreamInstanceActivityResponse.FlowCellHealth.ChannelStatePercentagesEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='ont.rpc.instance.StreamInstanceActivityResponse.FlowCellHealth.ChannelStatePercentagesEntry.value', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2259,
  serialized_end=2321,
)

_STREAMINSTANCEACTIVITYRESPONSE_FLOWCELLHEALTH = _descriptor.Descriptor(
  name='FlowCellHealth',
  full_name='ont.rpc.instance.StreamInstanceActivityResponse.FlowCellHealth',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='channel_state_percentages', full_name='ont.rpc.instance.StreamInstanceActivityResponse.FlowCellHealth.channel_state_percentages', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_STREAMINSTANCEACTIVITYRESPONSE_FLOWCELLHEALTH_CHANNELSTATEPERCENTAGESENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2112,
  serialized_end=2321,
)

_STREAMINSTANCEACTIVITYRESPONSE = _descriptor.Descriptor(
  name='StreamInstanceActivityResponse',
  full_name='ont.rpc.instance.StreamInstanceActivityResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='device_info', full_name='ont.rpc.instance.StreamInstanceActivityResponse.device_info', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='flow_cell_info', full_name='ont.rpc.instance.StreamInstanceActivityResponse.flow_cell_info', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='protocol_run_info', full_name='ont.rpc.instance.StreamInstanceActivityResponse.protocol_run_info', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='acquisition_run_info', full_name='ont.rpc.instance.StreamInstanceActivityResponse.acquisition_run_info', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='flow_cell_health', full_name='ont.rpc.instance.StreamInstanceActivityResponse.flow_cell_health', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='yield_summary', full_name='ont.rpc.instance.StreamInstanceActivityResponse.yield_summary', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_STREAMINSTANCEACTIVITYRESPONSE_FLOWCELLHEALTH, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='stream_value', full_name='ont.rpc.instance.StreamInstanceActivityResponse.stream_value',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=1656,
  serialized_end=2337,
)

_GETVERSIONINFORESPONSE_MINKNOWVERSION.containing_type = _GETVERSIONINFORESPONSE
_GETVERSIONINFORESPONSE.fields_by_name['minknow'].message_type = _GETVERSIONINFORESPONSE_MINKNOWVERSION
_GETVERSIONINFORESPONSE.fields_by_name['distribution_status'].enum_type = _GETVERSIONINFORESPONSE_DISTRIBUTIONSTATUS
_GETVERSIONINFORESPONSE_DISTRIBUTIONSTATUS.containing_type = _GETVERSIONINFORESPONSE
_GETDISKSPACEINFORESPONSE.fields_by_name['filesystem_disk_space_info'].message_type = _FILESYSTEMDISKSPACEINFO
_GETHOSTTYPERESPONSE.fields_by_name['host_type'].enum_type = _GETHOSTTYPERESPONSE_HOSTTYPE
_GETHOSTTYPERESPONSE_HOSTTYPE.containing_type = _GETHOSTTYPERESPONSE
_DEVICEINFO.fields_by_name['device_state'].enum_type = minknow_dot_rpc_dot_device__pb2._GETDEVICESTATERESPONSE_DEVICESTATE
_DEVICEINFO.fields_by_name['device_info'].message_type = minknow_dot_rpc_dot_device__pb2._GETDEVICEINFORESPONSE
_STREAMINSTANCEACTIVITYRESPONSE_FLOWCELLHEALTH_CHANNELSTATEPERCENTAGESENTRY.containing_type = _STREAMINSTANCEACTIVITYRESPONSE_FLOWCELLHEALTH
_STREAMINSTANCEACTIVITYRESPONSE_FLOWCELLHEALTH.fields_by_name['channel_state_percentages'].message_type = _STREAMINSTANCEACTIVITYRESPONSE_FLOWCELLHEALTH_CHANNELSTATEPERCENTAGESENTRY
_STREAMINSTANCEACTIVITYRESPONSE_FLOWCELLHEALTH.containing_type = _STREAMINSTANCEACTIVITYRESPONSE
_STREAMINSTANCEACTIVITYRESPONSE.fields_by_name['device_info'].message_type = _DEVICEINFO
_STREAMINSTANCEACTIVITYRESPONSE.fields_by_name['flow_cell_info'].message_type = minknow_dot_rpc_dot_device__pb2._GETFLOWCELLINFORESPONSE
_STREAMINSTANCEACTIVITYRESPONSE.fields_by_name['protocol_run_info'].message_type = minknow_dot_rpc_dot_protocol__pb2._PROTOCOLRUNINFO
_STREAMINSTANCEACTIVITYRESPONSE.fields_by_name['acquisition_run_info'].message_type = minknow_dot_rpc_dot_acquisition__pb2._ACQUISITIONRUNINFO
_STREAMINSTANCEACTIVITYRESPONSE.fields_by_name['flow_cell_health'].message_type = _STREAMINSTANCEACTIVITYRESPONSE_FLOWCELLHEALTH
_STREAMINSTANCEACTIVITYRESPONSE.fields_by_name['yield_summary'].message_type = minknow_dot_rpc_dot_acquisition__pb2._ACQUISITIONYIELDSUMMARY
_STREAMINSTANCEACTIVITYRESPONSE.oneofs_by_name['stream_value'].fields.append(
  _STREAMINSTANCEACTIVITYRESPONSE.fields_by_name['device_info'])
_STREAMINSTANCEACTIVITYRESPONSE.fields_by_name['device_info'].containing_oneof = _STREAMINSTANCEACTIVITYRESPONSE.oneofs_by_name['stream_value']
_STREAMINSTANCEACTIVITYRESPONSE.oneofs_by_name['stream_value'].fields.append(
  _STREAMINSTANCEACTIVITYRESPONSE.fields_by_name['flow_cell_info'])
_STREAMINSTANCEACTIVITYRESPONSE.fields_by_name['flow_cell_info'].containing_oneof = _STREAMINSTANCEACTIVITYRESPONSE.oneofs_by_name['stream_value']
_STREAMINSTANCEACTIVITYRESPONSE.oneofs_by_name['stream_value'].fields.append(
  _STREAMINSTANCEACTIVITYRESPONSE.fields_by_name['protocol_run_info'])
_STREAMINSTANCEACTIVITYRESPONSE.fields_by_name['protocol_run_info'].containing_oneof = _STREAMINSTANCEACTIVITYRESPONSE.oneofs_by_name['stream_value']
_STREAMINSTANCEACTIVITYRESPONSE.oneofs_by_name['stream_value'].fields.append(
  _STREAMINSTANCEACTIVITYRESPONSE.fields_by_name['acquisition_run_info'])
_STREAMINSTANCEACTIVITYRESPONSE.fields_by_name['acquisition_run_info'].containing_oneof = _STREAMINSTANCEACTIVITYRESPONSE.oneofs_by_name['stream_value']
_STREAMINSTANCEACTIVITYRESPONSE.oneofs_by_name['stream_value'].fields.append(
  _STREAMINSTANCEACTIVITYRESPONSE.fields_by_name['flow_cell_health'])
_STREAMINSTANCEACTIVITYRESPONSE.fields_by_name['flow_cell_health'].containing_oneof = _STREAMINSTANCEACTIVITYRESPONSE.oneofs_by_name['stream_value']
_STREAMINSTANCEACTIVITYRESPONSE.oneofs_by_name['stream_value'].fields.append(
  _STREAMINSTANCEACTIVITYRESPONSE.fields_by_name['yield_summary'])
_STREAMINSTANCEACTIVITYRESPONSE.fields_by_name['yield_summary'].containing_oneof = _STREAMINSTANCEACTIVITYRESPONSE.oneofs_by_name['stream_value']
DESCRIPTOR.message_types_by_name['GetVersionInfoRequest'] = _GETVERSIONINFOREQUEST
DESCRIPTOR.message_types_by_name['GetVersionInfoResponse'] = _GETVERSIONINFORESPONSE
DESCRIPTOR.message_types_by_name['GetOutputDirectoriesRequest'] = _GETOUTPUTDIRECTORIESREQUEST
DESCRIPTOR.message_types_by_name['OutputDirectories'] = _OUTPUTDIRECTORIES
DESCRIPTOR.message_types_by_name['GetDefaultOutputDirectoriesRequest'] = _GETDEFAULTOUTPUTDIRECTORIESREQUEST
DESCRIPTOR.message_types_by_name['SetOutputDirectoryRequest'] = _SETOUTPUTDIRECTORYREQUEST
DESCRIPTOR.message_types_by_name['SetOutputDirectoryResponse'] = _SETOUTPUTDIRECTORYRESPONSE
DESCRIPTOR.message_types_by_name['SetReadsDirectoryRequest'] = _SETREADSDIRECTORYREQUEST
DESCRIPTOR.message_types_by_name['SetReadsDirectoryResponse'] = _SETREADSDIRECTORYRESPONSE
DESCRIPTOR.message_types_by_name['FilesystemDiskSpaceInfo'] = _FILESYSTEMDISKSPACEINFO
DESCRIPTOR.message_types_by_name['GetDiskSpaceInfoRequest'] = _GETDISKSPACEINFOREQUEST
DESCRIPTOR.message_types_by_name['GetDiskSpaceInfoResponse'] = _GETDISKSPACEINFORESPONSE
DESCRIPTOR.message_types_by_name['GetMachineIdRequest'] = _GETMACHINEIDREQUEST
DESCRIPTOR.message_types_by_name['GetMachineIdResponse'] = _GETMACHINEIDRESPONSE
DESCRIPTOR.message_types_by_name['GetHostTypeRequest'] = _GETHOSTTYPEREQUEST
DESCRIPTOR.message_types_by_name['GetHostTypeResponse'] = _GETHOSTTYPERESPONSE
DESCRIPTOR.message_types_by_name['StreamInstanceActivityRequest'] = _STREAMINSTANCEACTIVITYREQUEST
DESCRIPTOR.message_types_by_name['DeviceInfo'] = _DEVICEINFO
DESCRIPTOR.message_types_by_name['StreamInstanceActivityResponse'] = _STREAMINSTANCEACTIVITYRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

GetVersionInfoRequest = _reflection.GeneratedProtocolMessageType('GetVersionInfoRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETVERSIONINFOREQUEST,
  '__module__' : 'minknow.rpc.instance_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.instance.GetVersionInfoRequest)
  })
_sym_db.RegisterMessage(GetVersionInfoRequest)

GetVersionInfoResponse = _reflection.GeneratedProtocolMessageType('GetVersionInfoResponse', (_message.Message,), {

  'MinknowVersion' : _reflection.GeneratedProtocolMessageType('MinknowVersion', (_message.Message,), {
    'DESCRIPTOR' : _GETVERSIONINFORESPONSE_MINKNOWVERSION,
    '__module__' : 'minknow.rpc.instance_pb2'
    # @@protoc_insertion_point(class_scope:ont.rpc.instance.GetVersionInfoResponse.MinknowVersion)
    })
  ,
  'DESCRIPTOR' : _GETVERSIONINFORESPONSE,
  '__module__' : 'minknow.rpc.instance_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.instance.GetVersionInfoResponse)
  })
_sym_db.RegisterMessage(GetVersionInfoResponse)
_sym_db.RegisterMessage(GetVersionInfoResponse.MinknowVersion)

GetOutputDirectoriesRequest = _reflection.GeneratedProtocolMessageType('GetOutputDirectoriesRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETOUTPUTDIRECTORIESREQUEST,
  '__module__' : 'minknow.rpc.instance_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.instance.GetOutputDirectoriesRequest)
  })
_sym_db.RegisterMessage(GetOutputDirectoriesRequest)

OutputDirectories = _reflection.GeneratedProtocolMessageType('OutputDirectories', (_message.Message,), {
  'DESCRIPTOR' : _OUTPUTDIRECTORIES,
  '__module__' : 'minknow.rpc.instance_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.instance.OutputDirectories)
  })
_sym_db.RegisterMessage(OutputDirectories)

GetDefaultOutputDirectoriesRequest = _reflection.GeneratedProtocolMessageType('GetDefaultOutputDirectoriesRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETDEFAULTOUTPUTDIRECTORIESREQUEST,
  '__module__' : 'minknow.rpc.instance_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.instance.GetDefaultOutputDirectoriesRequest)
  })
_sym_db.RegisterMessage(GetDefaultOutputDirectoriesRequest)

SetOutputDirectoryRequest = _reflection.GeneratedProtocolMessageType('SetOutputDirectoryRequest', (_message.Message,), {
  'DESCRIPTOR' : _SETOUTPUTDIRECTORYREQUEST,
  '__module__' : 'minknow.rpc.instance_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.instance.SetOutputDirectoryRequest)
  })
_sym_db.RegisterMessage(SetOutputDirectoryRequest)

SetOutputDirectoryResponse = _reflection.GeneratedProtocolMessageType('SetOutputDirectoryResponse', (_message.Message,), {
  'DESCRIPTOR' : _SETOUTPUTDIRECTORYRESPONSE,
  '__module__' : 'minknow.rpc.instance_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.instance.SetOutputDirectoryResponse)
  })
_sym_db.RegisterMessage(SetOutputDirectoryResponse)

SetReadsDirectoryRequest = _reflection.GeneratedProtocolMessageType('SetReadsDirectoryRequest', (_message.Message,), {
  'DESCRIPTOR' : _SETREADSDIRECTORYREQUEST,
  '__module__' : 'minknow.rpc.instance_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.instance.SetReadsDirectoryRequest)
  })
_sym_db.RegisterMessage(SetReadsDirectoryRequest)

SetReadsDirectoryResponse = _reflection.GeneratedProtocolMessageType('SetReadsDirectoryResponse', (_message.Message,), {
  'DESCRIPTOR' : _SETREADSDIRECTORYRESPONSE,
  '__module__' : 'minknow.rpc.instance_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.instance.SetReadsDirectoryResponse)
  })
_sym_db.RegisterMessage(SetReadsDirectoryResponse)

FilesystemDiskSpaceInfo = _reflection.GeneratedProtocolMessageType('FilesystemDiskSpaceInfo', (_message.Message,), {
  'DESCRIPTOR' : _FILESYSTEMDISKSPACEINFO,
  '__module__' : 'minknow.rpc.instance_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.instance.FilesystemDiskSpaceInfo)
  })
_sym_db.RegisterMessage(FilesystemDiskSpaceInfo)

GetDiskSpaceInfoRequest = _reflection.GeneratedProtocolMessageType('GetDiskSpaceInfoRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETDISKSPACEINFOREQUEST,
  '__module__' : 'minknow.rpc.instance_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.instance.GetDiskSpaceInfoRequest)
  })
_sym_db.RegisterMessage(GetDiskSpaceInfoRequest)

GetDiskSpaceInfoResponse = _reflection.GeneratedProtocolMessageType('GetDiskSpaceInfoResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETDISKSPACEINFORESPONSE,
  '__module__' : 'minknow.rpc.instance_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.instance.GetDiskSpaceInfoResponse)
  })
_sym_db.RegisterMessage(GetDiskSpaceInfoResponse)

GetMachineIdRequest = _reflection.GeneratedProtocolMessageType('GetMachineIdRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETMACHINEIDREQUEST,
  '__module__' : 'minknow.rpc.instance_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.instance.GetMachineIdRequest)
  })
_sym_db.RegisterMessage(GetMachineIdRequest)

GetMachineIdResponse = _reflection.GeneratedProtocolMessageType('GetMachineIdResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETMACHINEIDRESPONSE,
  '__module__' : 'minknow.rpc.instance_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.instance.GetMachineIdResponse)
  })
_sym_db.RegisterMessage(GetMachineIdResponse)

GetHostTypeRequest = _reflection.GeneratedProtocolMessageType('GetHostTypeRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETHOSTTYPEREQUEST,
  '__module__' : 'minknow.rpc.instance_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.instance.GetHostTypeRequest)
  })
_sym_db.RegisterMessage(GetHostTypeRequest)

GetHostTypeResponse = _reflection.GeneratedProtocolMessageType('GetHostTypeResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETHOSTTYPERESPONSE,
  '__module__' : 'minknow.rpc.instance_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.instance.GetHostTypeResponse)
  })
_sym_db.RegisterMessage(GetHostTypeResponse)

StreamInstanceActivityRequest = _reflection.GeneratedProtocolMessageType('StreamInstanceActivityRequest', (_message.Message,), {
  'DESCRIPTOR' : _STREAMINSTANCEACTIVITYREQUEST,
  '__module__' : 'minknow.rpc.instance_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.instance.StreamInstanceActivityRequest)
  })
_sym_db.RegisterMessage(StreamInstanceActivityRequest)

DeviceInfo = _reflection.GeneratedProtocolMessageType('DeviceInfo', (_message.Message,), {
  'DESCRIPTOR' : _DEVICEINFO,
  '__module__' : 'minknow.rpc.instance_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.instance.DeviceInfo)
  })
_sym_db.RegisterMessage(DeviceInfo)

StreamInstanceActivityResponse = _reflection.GeneratedProtocolMessageType('StreamInstanceActivityResponse', (_message.Message,), {

  'FlowCellHealth' : _reflection.GeneratedProtocolMessageType('FlowCellHealth', (_message.Message,), {

    'ChannelStatePercentagesEntry' : _reflection.GeneratedProtocolMessageType('ChannelStatePercentagesEntry', (_message.Message,), {
      'DESCRIPTOR' : _STREAMINSTANCEACTIVITYRESPONSE_FLOWCELLHEALTH_CHANNELSTATEPERCENTAGESENTRY,
      '__module__' : 'minknow.rpc.instance_pb2'
      # @@protoc_insertion_point(class_scope:ont.rpc.instance.StreamInstanceActivityResponse.FlowCellHealth.ChannelStatePercentagesEntry)
      })
    ,
    'DESCRIPTOR' : _STREAMINSTANCEACTIVITYRESPONSE_FLOWCELLHEALTH,
    '__module__' : 'minknow.rpc.instance_pb2'
    # @@protoc_insertion_point(class_scope:ont.rpc.instance.StreamInstanceActivityResponse.FlowCellHealth)
    })
  ,
  'DESCRIPTOR' : _STREAMINSTANCEACTIVITYRESPONSE,
  '__module__' : 'minknow.rpc.instance_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.instance.StreamInstanceActivityResponse)
  })
_sym_db.RegisterMessage(StreamInstanceActivityResponse)
_sym_db.RegisterMessage(StreamInstanceActivityResponse.FlowCellHealth)
_sym_db.RegisterMessage(StreamInstanceActivityResponse.FlowCellHealth.ChannelStatePercentagesEntry)


_STREAMINSTANCEACTIVITYRESPONSE_FLOWCELLHEALTH_CHANNELSTATEPERCENTAGESENTRY._options = None

_INSTANCESERVICE = _descriptor.ServiceDescriptor(
  name='InstanceService',
  full_name='ont.rpc.instance.InstanceService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=2340,
  serialized_end=3371,
  methods=[
  _descriptor.MethodDescriptor(
    name='get_version_info',
    full_name='ont.rpc.instance.InstanceService.get_version_info',
    index=0,
    containing_service=None,
    input_type=_GETVERSIONINFOREQUEST,
    output_type=_GETVERSIONINFORESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='get_output_directories',
    full_name='ont.rpc.instance.InstanceService.get_output_directories',
    index=1,
    containing_service=None,
    input_type=_GETOUTPUTDIRECTORIESREQUEST,
    output_type=_OUTPUTDIRECTORIES,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='get_default_output_directories',
    full_name='ont.rpc.instance.InstanceService.get_default_output_directories',
    index=2,
    containing_service=None,
    input_type=_GETDEFAULTOUTPUTDIRECTORIESREQUEST,
    output_type=_OUTPUTDIRECTORIES,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='set_output_directory',
    full_name='ont.rpc.instance.InstanceService.set_output_directory',
    index=3,
    containing_service=None,
    input_type=_SETOUTPUTDIRECTORYREQUEST,
    output_type=_SETOUTPUTDIRECTORYRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='set_reads_directory',
    full_name='ont.rpc.instance.InstanceService.set_reads_directory',
    index=4,
    containing_service=None,
    input_type=_SETREADSDIRECTORYREQUEST,
    output_type=_SETREADSDIRECTORYRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='get_disk_space_info',
    full_name='ont.rpc.instance.InstanceService.get_disk_space_info',
    index=5,
    containing_service=None,
    input_type=_GETDISKSPACEINFOREQUEST,
    output_type=_GETDISKSPACEINFORESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='get_machine_id',
    full_name='ont.rpc.instance.InstanceService.get_machine_id',
    index=6,
    containing_service=None,
    input_type=_GETMACHINEIDREQUEST,
    output_type=_GETMACHINEIDRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='get_host_type',
    full_name='ont.rpc.instance.InstanceService.get_host_type',
    index=7,
    containing_service=None,
    input_type=_GETHOSTTYPEREQUEST,
    output_type=_GETHOSTTYPERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='stream_instance_activity',
    full_name='ont.rpc.instance.InstanceService.stream_instance_activity',
    index=8,
    containing_service=None,
    input_type=_STREAMINSTANCEACTIVITYREQUEST,
    output_type=_STREAMINSTANCEACTIVITYRESPONSE,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_INSTANCESERVICE)

DESCRIPTOR.services_by_name['InstanceService'] = _INSTANCESERVICE

# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: minknow/rpc/rpc_options.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import descriptor_pb2 as google_dot_protobuf_dot_descriptor__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='minknow/rpc/rpc_options.proto',
  package='ont.rpc',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x1dminknow/rpc/rpc_options.proto\x12\x07ont.rpc\x1a google/protobuf/descriptor.proto:5\n\x0crpc_required\x12\x1d.google.protobuf.FieldOptions\x18\xd1\x86\x03 \x01(\x08:3\n\nrpc_unwrap\x12\x1d.google.protobuf.FieldOptions\x18\xd2\x86\x03 \x01(\x08\x62\x06proto3')
  ,
  dependencies=[google_dot_protobuf_dot_descriptor__pb2.DESCRIPTOR,])


RPC_REQUIRED_FIELD_NUMBER = 50001
rpc_required = _descriptor.FieldDescriptor(
  name='rpc_required', full_name='ont.rpc.rpc_required', index=0,
  number=50001, type=8, cpp_type=7, label=1,
  has_default_value=False, default_value=False,
  message_type=None, enum_type=None, containing_type=None,
  is_extension=True, extension_scope=None,
  serialized_options=None, file=DESCRIPTOR)
RPC_UNWRAP_FIELD_NUMBER = 50002
rpc_unwrap = _descriptor.FieldDescriptor(
  name='rpc_unwrap', full_name='ont.rpc.rpc_unwrap', index=1,
  number=50002, type=8, cpp_type=7, label=1,
  has_default_value=False, default_value=False,
  message_type=None, enum_type=None, containing_type=None,
  is_extension=True, extension_scope=None,
  serialized_options=None, file=DESCRIPTOR)

DESCRIPTOR.extensions_by_name['rpc_required'] = rpc_required
DESCRIPTOR.extensions_by_name['rpc_unwrap'] = rpc_unwrap
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

google_dot_protobuf_dot_descriptor__pb2.FieldOptions.RegisterExtension(rpc_required)
google_dot_protobuf_dot_descriptor__pb2.FieldOptions.RegisterExtension(rpc_unwrap)

# @@protoc_insertion_point(module_scope)



from unittest import TestCase





class TestPerformance(TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def testPerformanceOnNFiles(self, n=3000):
        pass


# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from porerefiner.protocols.porerefiner.rpc import porerefiner_pb2 as porerefiner_dot_protocols_dot_porerefiner_dot_rpc_dot_porerefiner__pb2


class PoreRefinerStub(object):
    """Missing associated documentation comment in .proto file"""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetRuns = channel.unary_unary(
                '/porerefiner.rpc.PoreRefiner/GetRuns',
                request_serializer=porerefiner_dot_protocols_dot_porerefiner_dot_rpc_dot_porerefiner__pb2.RunListRequest.SerializeToString,
                response_deserializer=porerefiner_dot_protocols_dot_porerefiner_dot_rpc_dot_porerefiner__pb2.RunListResponse.FromString,
                )
        self.GetRunInfo = channel.unary_unary(
                '/porerefiner.rpc.PoreRefiner/GetRunInfo',
                request_serializer=porerefiner_dot_protocols_dot_porerefiner_dot_rpc_dot_porerefiner__pb2.RunRequest.SerializeToString,
                response_deserializer=porerefiner_dot_protocols_dot_porerefiner_dot_rpc_dot_porerefiner__pb2.RunResponse.FromString,
                )
        self.AttachSheetToRun = channel.unary_unary(
                '/porerefiner.rpc.PoreRefiner/AttachSheetToRun',
                request_serializer=porerefiner_dot_protocols_dot_porerefiner_dot_rpc_dot_porerefiner__pb2.RunAttachRequest.SerializeToString,
                response_deserializer=porerefiner_dot_protocols_dot_porerefiner_dot_rpc_dot_porerefiner__pb2.GenericResponse.FromString,
                )
        self.RsyncRunTo = channel.unary_unary(
                '/porerefiner.rpc.PoreRefiner/RsyncRunTo',
                request_serializer=porerefiner_dot_protocols_dot_porerefiner_dot_rpc_dot_porerefiner__pb2.RunRsyncRequest.SerializeToString,
                response_deserializer=porerefiner_dot_protocols_dot_porerefiner_dot_rpc_dot_porerefiner__pb2.RunRsyncResponse.FromString,
                )
        self.Tag = channel.unary_unary(
                '/porerefiner.rpc.PoreRefiner/Tag',
                request_serializer=porerefiner_dot_protocols_dot_porerefiner_dot_rpc_dot_porerefiner__pb2.TagRequest.SerializeToString,
                response_deserializer=porerefiner_dot_protocols_dot_porerefiner_dot_rpc_dot_porerefiner__pb2.GenericResponse.FromString,
                )


class PoreRefinerServicer(object):
    """Missing associated documentation comment in .proto file"""

    def GetRuns(self, request, context):
        """Missing associated documentation comment in .proto file"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetRunInfo(self, request, context):
        """Missing associated documentation comment in .proto file"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AttachSheetToRun(self, request, context):
        """Missing associated documentation comment in .proto file"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RsyncRunTo(self, request, context):
        """Missing associated documentation comment in .proto file"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Tag(self, request, context):
        """Missing associated documentation comment in .proto file"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_PoreRefinerServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'GetRuns': grpc.unary_unary_rpc_method_handler(
                    servicer.GetRuns,
                    request_deserializer=porerefiner_dot_protocols_dot_porerefiner_dot_rpc_dot_porerefiner__pb2.RunListRequest.FromString,
                    response_serializer=porerefiner_dot_protocols_dot_porerefiner_dot_rpc_dot_porerefiner__pb2.RunListResponse.SerializeToString,
            ),
            'GetRunInfo': grpc.unary_unary_rpc_method_handler(
                    servicer.GetRunInfo,
                    request_deserializer=porerefiner_dot_protocols_dot_porerefiner_dot_rpc_dot_porerefiner__pb2.RunRequest.FromString,
                    response_serializer=porerefiner_dot_protocols_dot_porerefiner_dot_rpc_dot_porerefiner__pb2.RunResponse.SerializeToString,
            ),
            'AttachSheetToRun': grpc.unary_unary_rpc_method_handler(
                    servicer.AttachSheetToRun,
                    request_deserializer=porerefiner_dot_protocols_dot_porerefiner_dot_rpc_dot_porerefiner__pb2.RunAttachRequest.FromString,
                    response_serializer=porerefiner_dot_protocols_dot_porerefiner_dot_rpc_dot_porerefiner__pb2.GenericResponse.SerializeToString,
            ),
            'RsyncRunTo': grpc.unary_unary_rpc_method_handler(
                    servicer.RsyncRunTo,
                    request_deserializer=porerefiner_dot_protocols_dot_porerefiner_dot_rpc_dot_porerefiner__pb2.RunRsyncRequest.FromString,
                    response_serializer=porerefiner_dot_protocols_dot_porerefiner_dot_rpc_dot_porerefiner__pb2.RunRsyncResponse.SerializeToString,
            ),
            'Tag': grpc.unary_unary_rpc_method_handler(
                    servicer.Tag,
                    request_deserializer=porerefiner_dot_protocols_dot_porerefiner_dot_rpc_dot_porerefiner__pb2.TagRequest.FromString,
                    response_serializer=porerefiner_dot_protocols_dot_porerefiner_dot_rpc_dot_porerefiner__pb2.GenericResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'porerefiner.rpc.PoreRefiner', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class PoreRefiner(object):
    """Missing associated documentation comment in .proto file"""

    @staticmethod
    def GetRuns(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/porerefiner.rpc.PoreRefiner/GetRuns',
            porerefiner_dot_protocols_dot_porerefiner_dot_rpc_dot_porerefiner__pb2.RunListRequest.SerializeToString,
            porerefiner_dot_protocols_dot_porerefiner_dot_rpc_dot_porerefiner__pb2.RunListResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetRunInfo(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/porerefiner.rpc.PoreRefiner/GetRunInfo',
            porerefiner_dot_protocols_dot_porerefiner_dot_rpc_dot_porerefiner__pb2.RunRequest.SerializeToString,
            porerefiner_dot_protocols_dot_porerefiner_dot_rpc_dot_porerefiner__pb2.RunResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AttachSheetToRun(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/porerefiner.rpc.PoreRefiner/AttachSheetToRun',
            porerefiner_dot_protocols_dot_porerefiner_dot_rpc_dot_porerefiner__pb2.RunAttachRequest.SerializeToString,
            porerefiner_dot_protocols_dot_porerefiner_dot_rpc_dot_porerefiner__pb2.GenericResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RsyncRunTo(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/porerefiner.rpc.PoreRefiner/RsyncRunTo',
            porerefiner_dot_protocols_dot_porerefiner_dot_rpc_dot_porerefiner__pb2.RunRsyncRequest.SerializeToString,
            porerefiner_dot_protocols_dot_porerefiner_dot_rpc_dot_porerefiner__pb2.RunRsyncResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Tag(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/porerefiner.rpc.PoreRefiner/Tag',
            porerefiner_dot_protocols_dot_porerefiner_dot_rpc_dot_porerefiner__pb2.TagRequest.SerializeToString,
            porerefiner_dot_protocols_dot_porerefiner_dot_rpc_dot_porerefiner__pb2.GenericResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)



from porerefiner.protocols.porerefiner.rpc.porerefiner_pb2 import SampleSheet

import csv
from io import TextIOWrapper

# We should do sample sheet parsing on the client side

def load_from_csv(file, delimiter=b',') -> SampleSheet:
    ss = SampleSheet()
    _, ss.porerefiner_ver, *_ = file.readline().strip().split(delimiter)
    ss.date.GetCurrentTime()
    if ss.porerefiner_ver == '1.0.0':
        _, ss.library_id, *_ = file.readline().strip().split(delimiter)
        _, ss.sequencing_kit, *_ = file.readline().strip().split(delimiter)
        delimiter = delimiter.decode()
        [ss.samples.add(**row) for row in csv.DictReader(TextIOWrapper(file), delimiter=delimiter, dialect='excel')] #this should handle commas in fields
    elif ss.porerefiner_ver == '1.0.0-fda':
        _, ss.library_id, *_ = file.readline().strip().split(delimiter)
        _, ss.sequencing_kit, *_ = file.readline().strip().split(delimiter)
        delimiter = delimiter.decode()
        file.readline() #ditch the header
        for sample_id, accession, barcode_id, organism, extraction_kit, comment, user, *_ in csv.reader(TextIOWrapper(file), delimiter=delimiter, dialect='excel'):
            ss.samples.add(sample_id=sample_id,
                           accession=accession,
                           barcode_id=barcode_id,
                           organism=organism,
                           extraction_kit=extraction_kit,
                           comment=comment,
                           user=user)
    elif ss.porerefiner_ver == '1.0.1':
        _, ss.library_id, *_ = file.readline().strip().split(delimiter)
        _, ss.sequencing_kit, *_ = file.readline().strip().split(delimiter)
        # _, ss.barcode_kit, *_ = file.readline().split(delimiter) #TODO
        # barcode kit takes multiple values
        _, *barcodes = file.readline().strip().split(delimiter)
        [ss.barcode_kit.append(barcode) for barcode in barcodes if barcode]
        delimiter = delimiter.decode()
        file.readline() # ditch the header
        for sample_id, accession, barcode_id, organism, extraction_kit, comment, user, *_ in csv.reader(TextIOWrapper(file), delimiter=delimiter, dialect='excel'):
            ss.samples.add(sample_id=sample_id,
                           accession=accession,
                           barcode_id=barcode_id,
                           organism=organism,
                           extraction_kit=extraction_kit,
                           comment=comment,
                           user=user)
    else:
        raise ValueError(f"Sample sheet of version {ss.porerefiner_ver} not supported.")
    return ss

def load_from_excel(file) -> SampleSheet:
    import openpyxl
    ss = SampleSheet()
    book = openpyxl.load_workbook(file)
    rows = (tuple(c.value for c in row) for row in book.worksheets[0].iter_rows())
    _, ss.porerefiner_ver, *_ = next(rows)
    if ss.porerefiner_ver == '1.0.0' or ss.porerefiner_ver == '1.0.0-fda':
        ss.date.GetCurrentTime()
        _, ss.library_id, *_ = next(rows)
        _, ss.sequencing_kit, *_ = next(rows)
        next(rows) # ditch the header
        for sample_id, accession, barcode_id, organism, extraction_kit, comment, user, *_ in rows:
            ss.samples.add(sample_id=sample_id,
                           accession=accession,
                           barcode_id=str(barcode_id),
                           organism=organism,
                           extraction_kit=extraction_kit,
                           comment=comment,
                           user=user)
    elif ss.porerefiner_ver == '1.0.1':
        ss.date.GetCurrentTime()
        _, ss.library_id, *_ = next(rows)
        _, ss.sequencing_kit, *_ = next(rows)
        _, ss.barcode_kit, *_ = next(rows)
        next(rows) # ditch the header
        for sample_id, accession, barcode_id, organism, extraction_kit, comment, user, *_ in rows:
            ss.samples.add(sample_id=sample_id,
                           accession=accession,
                           barcode_id=str(barcode_id),
                           organism=organism,
                           extraction_kit=extraction_kit,
                           comment=comment,
                           user=user)

    else:
        raise ValueError(f"Sample sheet of version {ss.porerefiner_ver} not supported.")

    return ss


from unittest import TestCase, skip

import porerefiner.jobs as jobs
import porerefiner.jobs.submitters as submitters

from tests import TestJob, TestSubmitter

from dataclasses import dataclass


class TestJobBaseClasses(TestCase):

    def test_class_import(self):
        self.assertGreater(len(jobs.CLASS_REGISTRY), 1)

    
    def test_file_job_base_class(self):
        @dataclass
        class TestFileJob(jobs.FileJob):
            field: str
            def setup(*a, **k):
                pass
            def collect(*a, **k):
                pass
        t = TestFileJob(submitter=TestSubmitter(), field="")
        assert t

    
    def test_run_job_base_class(self):
        @dataclass
        class TestRunJob(jobs.RunJob):
            field: str
            def setup(*a, **k):
                pass
            def collect(*a, **k):
                pass
        t = TestRunJob(submitter=TestSubmitter(), field="")
        assert t

    @skip('not yet implemented')
    def test_job_subclassing(self):
        assert False

    @skip("jobs aren't singletons anymore")
    def test_job_singleton(self):
        job = TestJob(submitter=TestSubmitter()) # init the class
        assert job._the_instance == job
        # assert TestJob._the_instance == job # prove that the class retains a reference to the singleton
        assert job == jobs.REGISTRY['TestJob'] # prove that you can now look up the singleton
        # assert job == jobs.REGISTRY['TestJob'](submitter=None) # prove that its not re-initialized
        assert job.submitter != None # prove that state didn't change


class TestSubmitterBaseClass(TestCase):

    # @skip('not yet implemented')
    def test_submitter(self):
        t = TestSubmitter()
        assert t
        assert t in submitters.SUBMITTERS



import logging
from contextlib import contextmanager

log = logging.getLogger('porerefiner.daemon')

try:
    import daemon
except ModuleNotFoundError:
    log.critical("Couldn't import python-daemon.")
    log.critical("Might be running under Windows where daemonization isn't supported.")

    class HalfDaemon:

        @contextmanager
        def DaemonContext(self):
            raise NotImplementedError("Daemonization not supported under Windows.")
            yield

    daemon = HalfDaemon()


#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = ['Click>=7.0',
                'flask>=1.1.1',
                'peewee>=3.13.3',
                'gunicorn>=19.9.0',
                'watchdog>=0.9.0',
                'hachiko>=0.2.0',
                'aiohttp>=3.6.1',
                'aiofile>=3.0.0',
                'namesgenerator>=0.3',
                'python-daemon>=2.2.3',
                'protobuf>=3.10.0',
                'grpclib>=0.3.0',
                'tabulate>=0.8.5',
                'asyncssh>=2.0.1',
                'setproctitle>=1.1.10',
                'pyyaml>=5.3'
                ]

setup_requirements = [ ]

test_requirements = [ 'hypothesis>=4.57.1',
                      'hypothesis-fspaths>=0.1',
                      'Mock>=4.0.0',
                      'aiounittest>=1.3.0']

setup(
    author="Justin Payne",
    author_email='justin.payne@fda.hhs.gov',
    python_requires='>=3.7',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.7',
    ],
    description="To help you manage your pores",
    entry_points={
        'console_scripts': [
            'prfr=porerefiner.cli:cli',
            'porerefinerd=porerefiner.porerefiner:cli'
        ],
        'porerefiner.plugins':""
    },
    install_requires=requirements,
    license="MIT license",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='porerefiner',
    name='porerefiner',
    packages=find_packages(include=['porerefiner', 'porerefiner.*']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/CFSAN-Biostatistics/porerefiner',
    version='0.8.18',
    zip_safe=False,
)

# from pathlib import Path
# from shutil import copy

# installdir = Path(__file__).parent

# copy(installdir / 'porerefiner' / 'porerefiner.service',
#      Path.home() / '.config' / 'systemd' / 'user.control' / 'porerefiner.service')

# copy(installdir / 'porerefiner' / 'porerefiner.app.service',
#      Path.home() / '.config' / 'systemd' / 'user.control' / 'porerefiner.app.service')



from dataclasses import dataclass
from asyncssh import connect
from pathlib import Path
from typing import Union


from porerefiner.jobs.submitters import Submitter, Url, Email, Path

import asyncio
import subprocess

@dataclass
class HpcSubmitter(Submitter):
    "Job submitter for interfacing with HPC resources."

    login_host: str
    username: str
    private_key_path: str
    known_hosts_path: str
    scheduler: str = "uge"
    queue: str = "long.q"
    remote_root: str = "~"

    async def send(self, cmd):
        async with connect(self.login_host,
                        username=self.username,
                        client_keys=[self.private_key_path],
                        known_hosts=self.known_hosts_path) as conn:
            return await conn.run(cmd)

    async def test_noop(self):
        subprocess.run(['rsync', '--version']).check_returncode()
        self.remote_root = Path((await self.send('python -c "import tempfile; print(tempfile.gettempdir())"')).strip())


    def reroot_path(self, path):
        return Path(self.remote_root, path)

    async def begin_job(self, command, datadir, remotedir, environment_hints={}):
        hints = " ".join([f"{name}={value}" for name, value in environment_hints.items()])
        return await self.send(f'''echo "{hints} {command}" | qsub -q {self.queue}''')

    async def poll_job(self, job):
        result = await self.send(f"qacct")

    def closeout_job(self, job, datadir, remotedir):
        pass




from dataclasses import dataclass
from porerefiner.jobs.submitters import Submitter
from asyncio import subprocess

processes = {}


@dataclass
class LocalSubmitter(Submitter):

    async def test_noop(self):
        pass

    def reroot_path(self, path):
        return path #no op, no need to reroot paths

    async def begin_job(self, command, datadir, remotedir, environment_hints={}):
        processes[datadir] = await subprocess.create_subprocess_shell(command)

    async def poll_job(self, job):
        await processes[job.datadir].wait()
        if processes[job.datadir].returncode:
            return 'FAILED'
        return 'DONE'

    def closeout_job(self, job, datadir, remotedir):
        del processes[datadir]





import asyncio
import logging
from dataclasses import dataclass

from porerefiner.notifiers import Notifier

log = logging.getLogger('porerefiner.toast_notifier')

@dataclass
class ToastNotifier(Notifier): #TODO
    "Notifier that throws up an OS 'toast' on Windows, Ubuntu, and CentOS"

    async def notify(self, run, state, message):
        try:
            from pynotifier import Notification
            Notification(title=message,
                         description=f"run {run.alt_name} ({run.path}) completed.",
                         duration=10).send()
        except ImportError:
            log.error(f"Toast notifier configured but py-notifier not installed. Please install py-notifier (pip install py-notifier) for OS toast notifications.")
        except SystemError:
            log.error(f"Toast notifier configured but py-notifier doesn't work under Mac OS X.")



import asyncio
import pkgutil
# from ..models import PorerefinerModel

from porerefiner.models import Run
from porerefiner.cli_utils import render_dataclass, Email, Url, PathStr

from typing import Any

# states = PorerefinerModel.statuses

REGISTRY = {}


def _register_class(target_class):
    REGISTRY[target_class.__name__] = target_class

class _MetaRegistry(type):

    def __new__(meta, name, bases, class_dict):
        cls = type.__new__(meta, name, bases, class_dict)
        if name not in REGISTRY:
            _register_class(cls)
        return cls

    def __call__(cls, *args, **kwargs):
        the_instance = super().__call__(*args, **kwargs)
        NOTIFIERS.append(the_instance)
        return the_instance


class Notifier(metaclass=_MetaRegistry):
    "Abstract, non-configurable base class for notifiers"

    def __init__(self, name, *args, **kwargs):
        super().__init__()
        self.name = name

    async def notify(self, run: Run, state: Any, message: str):
        raise NotImplementedError('Notifier not implemented.')

    @classmethod
    def get_configurable_options(cls):
        "Enumerate configurable options and value type as a guide to configuration."
        return render_dataclass(cls)



NOTIFIERS = []

# from . import galaxy
# from . import http
# from . import sqs
# from . import toast

for loader, module_name, is_pkg in  pkgutil.walk_packages(__path__):
    _module = loader.find_module(module_name).load_module(module_name)
    globals()[module_name] = _module




# Generated by the Protocol Buffers compiler. DO NOT EDIT!
# source: minknow/rpc/protocol.proto
# plugin: grpclib.plugin.main
import abc
import typing

import grpclib.const
import grpclib.client
if typing.TYPE_CHECKING:
    import grpclib.server

from . import acquisition_pb2
from . import device_pb2
from . import rpc_options_pb2
import google.protobuf.timestamp_pb2
import google.protobuf.wrappers_pb2
from . import protocol_pb2


class ProtocolServiceBase(abc.ABC):

    @abc.abstractmethod
    async def start_protocol(self, stream: 'grpclib.server.Stream[minknow.rpc.protocol_pb2.StartProtocolRequest, minknow.rpc.protocol_pb2.StartProtocolResponse]') -> None:
        pass

    @abc.abstractmethod
    async def stop_protocol(self, stream: 'grpclib.server.Stream[minknow.rpc.protocol_pb2.StopProtocolRequest, minknow.rpc.protocol_pb2.StopProtocolResponse]') -> None:
        pass

    @abc.abstractmethod
    async def wait_for_finished(self, stream: 'grpclib.server.Stream[minknow.rpc.protocol_pb2.WaitForFinishedRequest, minknow.rpc.protocol_pb2.ProtocolRunInfo]') -> None:
        pass

    @abc.abstractmethod
    async def get_run_info(self, stream: 'grpclib.server.Stream[minknow.rpc.protocol_pb2.GetRunInfoRequest, minknow.rpc.protocol_pb2.ProtocolRunInfo]') -> None:
        pass

    @abc.abstractmethod
    async def list_protocol_runs(self, stream: 'grpclib.server.Stream[minknow.rpc.protocol_pb2.ListProtocolRunsRequest, minknow.rpc.protocol_pb2.ListProtocolRunsResponse]') -> None:
        pass

    @abc.abstractmethod
    async def get_current_protocol_run(self, stream: 'grpclib.server.Stream[minknow.rpc.protocol_pb2.GetCurrentProtocolRunRequest, minknow.rpc.protocol_pb2.ProtocolRunInfo]') -> None:
        pass

    @abc.abstractmethod
    async def watch_current_protocol_run(self, stream: 'grpclib.server.Stream[minknow.rpc.protocol_pb2.WatchCurrentProtocolRunRequest, minknow.rpc.protocol_pb2.ProtocolRunInfo]') -> None:
        pass

    @abc.abstractmethod
    async def list_protocols(self, stream: 'grpclib.server.Stream[minknow.rpc.protocol_pb2.ListProtocolsRequest, minknow.rpc.protocol_pb2.ListProtocolsResponse]') -> None:
        pass

    @abc.abstractmethod
    async def get_context_info(self, stream: 'grpclib.server.Stream[minknow.rpc.protocol_pb2.GetContextInfoRequest, minknow.rpc.protocol_pb2.GetContextInfoResponse]') -> None:
        pass

    @abc.abstractmethod
    async def set_context_info(self, stream: 'grpclib.server.Stream[minknow.rpc.protocol_pb2.SetContextInfoRequest, minknow.rpc.protocol_pb2.SetContextInfoResponse]') -> None:
        pass

    @abc.abstractmethod
    async def get_sample_id(self, stream: 'grpclib.server.Stream[minknow.rpc.protocol_pb2.GetSampleIdRequest, minknow.rpc.protocol_pb2.GetSampleIdResponse]') -> None:
        pass

    @abc.abstractmethod
    async def set_sample_id(self, stream: 'grpclib.server.Stream[minknow.rpc.protocol_pb2.SetSampleIdRequest, minknow.rpc.protocol_pb2.SetSampleIdResponse]') -> None:
        pass

    @abc.abstractmethod
    async def get_protocol_purpose(self, stream: 'grpclib.server.Stream[minknow.rpc.protocol_pb2.GetProtocolPurposeRequest, minknow.rpc.protocol_pb2.GetProtocolPurposeResponse]') -> None:
        pass

    @abc.abstractmethod
    async def set_protocol_purpose(self, stream: 'grpclib.server.Stream[minknow.rpc.protocol_pb2.SetProtocolPurposeRequest, minknow.rpc.protocol_pb2.SetProtocolPurposeResponse]') -> None:
        pass

    @abc.abstractmethod
    async def add_epi2me_workflow(self, stream: 'grpclib.server.Stream[minknow.rpc.protocol_pb2.AddEpi2meWorkflowRequest, minknow.rpc.protocol_pb2.AddEpi2meWorkflowResponse]') -> None:
        pass

    @abc.abstractmethod
    async def list_protocol_group_ids(self, stream: 'grpclib.server.Stream[minknow.rpc.protocol_pb2.ListProtocolGroupIdsRequest, minknow.rpc.protocol_pb2.ListProtocolGroupIdsResponse]') -> None:
        pass

    def __mapping__(self) -> typing.Dict[str, grpclib.const.Handler]:
        return {
            '/ont.rpc.protocol.ProtocolService/start_protocol': grpclib.const.Handler(
                self.start_protocol,
                grpclib.const.Cardinality.UNARY_UNARY,
                minknow.rpc.protocol_pb2.StartProtocolRequest,
                minknow.rpc.protocol_pb2.StartProtocolResponse,
            ),
            '/ont.rpc.protocol.ProtocolService/stop_protocol': grpclib.const.Handler(
                self.stop_protocol,
                grpclib.const.Cardinality.UNARY_UNARY,
                minknow.rpc.protocol_pb2.StopProtocolRequest,
                minknow.rpc.protocol_pb2.StopProtocolResponse,
            ),
            '/ont.rpc.protocol.ProtocolService/wait_for_finished': grpclib.const.Handler(
                self.wait_for_finished,
                grpclib.const.Cardinality.UNARY_UNARY,
                minknow.rpc.protocol_pb2.WaitForFinishedRequest,
                minknow.rpc.protocol_pb2.ProtocolRunInfo,
            ),
            '/ont.rpc.protocol.ProtocolService/get_run_info': grpclib.const.Handler(
                self.get_run_info,
                grpclib.const.Cardinality.UNARY_UNARY,
                minknow.rpc.protocol_pb2.GetRunInfoRequest,
                minknow.rpc.protocol_pb2.ProtocolRunInfo,
            ),
            '/ont.rpc.protocol.ProtocolService/list_protocol_runs': grpclib.const.Handler(
                self.list_protocol_runs,
                grpclib.const.Cardinality.UNARY_UNARY,
                minknow.rpc.protocol_pb2.ListProtocolRunsRequest,
                minknow.rpc.protocol_pb2.ListProtocolRunsResponse,
            ),
            '/ont.rpc.protocol.ProtocolService/get_current_protocol_run': grpclib.const.Handler(
                self.get_current_protocol_run,
                grpclib.const.Cardinality.UNARY_UNARY,
                minknow.rpc.protocol_pb2.GetCurrentProtocolRunRequest,
                minknow.rpc.protocol_pb2.ProtocolRunInfo,
            ),
            '/ont.rpc.protocol.ProtocolService/watch_current_protocol_run': grpclib.const.Handler(
                self.watch_current_protocol_run,
                grpclib.const.Cardinality.UNARY_STREAM,
                minknow.rpc.protocol_pb2.WatchCurrentProtocolRunRequest,
                minknow.rpc.protocol_pb2.ProtocolRunInfo,
            ),
            '/ont.rpc.protocol.ProtocolService/list_protocols': grpclib.const.Handler(
                self.list_protocols,
                grpclib.const.Cardinality.UNARY_UNARY,
                minknow.rpc.protocol_pb2.ListProtocolsRequest,
                minknow.rpc.protocol_pb2.ListProtocolsResponse,
            ),
            '/ont.rpc.protocol.ProtocolService/get_context_info': grpclib.const.Handler(
                self.get_context_info,
                grpclib.const.Cardinality.UNARY_UNARY,
                minknow.rpc.protocol_pb2.GetContextInfoRequest,
                minknow.rpc.protocol_pb2.GetContextInfoResponse,
            ),
            '/ont.rpc.protocol.ProtocolService/set_context_info': grpclib.const.Handler(
                self.set_context_info,
                grpclib.const.Cardinality.UNARY_UNARY,
                minknow.rpc.protocol_pb2.SetContextInfoRequest,
                minknow.rpc.protocol_pb2.SetContextInfoResponse,
            ),
            '/ont.rpc.protocol.ProtocolService/get_sample_id': grpclib.const.Handler(
                self.get_sample_id,
                grpclib.const.Cardinality.UNARY_UNARY,
                minknow.rpc.protocol_pb2.GetSampleIdRequest,
                minknow.rpc.protocol_pb2.GetSampleIdResponse,
            ),
            '/ont.rpc.protocol.ProtocolService/set_sample_id': grpclib.const.Handler(
                self.set_sample_id,
                grpclib.const.Cardinality.UNARY_UNARY,
                minknow.rpc.protocol_pb2.SetSampleIdRequest,
                minknow.rpc.protocol_pb2.SetSampleIdResponse,
            ),
            '/ont.rpc.protocol.ProtocolService/get_protocol_purpose': grpclib.const.Handler(
                self.get_protocol_purpose,
                grpclib.const.Cardinality.UNARY_UNARY,
                minknow.rpc.protocol_pb2.GetProtocolPurposeRequest,
                minknow.rpc.protocol_pb2.GetProtocolPurposeResponse,
            ),
            '/ont.rpc.protocol.ProtocolService/set_protocol_purpose': grpclib.const.Handler(
                self.set_protocol_purpose,
                grpclib.const.Cardinality.UNARY_UNARY,
                minknow.rpc.protocol_pb2.SetProtocolPurposeRequest,
                minknow.rpc.protocol_pb2.SetProtocolPurposeResponse,
            ),
            '/ont.rpc.protocol.ProtocolService/add_epi2me_workflow': grpclib.const.Handler(
                self.add_epi2me_workflow,
                grpclib.const.Cardinality.UNARY_UNARY,
                minknow.rpc.protocol_pb2.AddEpi2meWorkflowRequest,
                minknow.rpc.protocol_pb2.AddEpi2meWorkflowResponse,
            ),
            '/ont.rpc.protocol.ProtocolService/list_protocol_group_ids': grpclib.const.Handler(
                self.list_protocol_group_ids,
                grpclib.const.Cardinality.UNARY_UNARY,
                minknow.rpc.protocol_pb2.ListProtocolGroupIdsRequest,
                minknow.rpc.protocol_pb2.ListProtocolGroupIdsResponse,
            ),
        }


class ProtocolServiceStub:

    def __init__(self, channel: grpclib.client.Channel) -> None:
        self.start_protocol = grpclib.client.UnaryUnaryMethod(
            channel,
            '/ont.rpc.protocol.ProtocolService/start_protocol',
            minknow.rpc.protocol_pb2.StartProtocolRequest,
            minknow.rpc.protocol_pb2.StartProtocolResponse,
        )
        self.stop_protocol = grpclib.client.UnaryUnaryMethod(
            channel,
            '/ont.rpc.protocol.ProtocolService/stop_protocol',
            minknow.rpc.protocol_pb2.StopProtocolRequest,
            minknow.rpc.protocol_pb2.StopProtocolResponse,
        )
        self.wait_for_finished = grpclib.client.UnaryUnaryMethod(
            channel,
            '/ont.rpc.protocol.ProtocolService/wait_for_finished',
            minknow.rpc.protocol_pb2.WaitForFinishedRequest,
            minknow.rpc.protocol_pb2.ProtocolRunInfo,
        )
        self.get_run_info = grpclib.client.UnaryUnaryMethod(
            channel,
            '/ont.rpc.protocol.ProtocolService/get_run_info',
            minknow.rpc.protocol_pb2.GetRunInfoRequest,
            minknow.rpc.protocol_pb2.ProtocolRunInfo,
        )
        self.list_protocol_runs = grpclib.client.UnaryUnaryMethod(
            channel,
            '/ont.rpc.protocol.ProtocolService/list_protocol_runs',
            minknow.rpc.protocol_pb2.ListProtocolRunsRequest,
            minknow.rpc.protocol_pb2.ListProtocolRunsResponse,
        )
        self.get_current_protocol_run = grpclib.client.UnaryUnaryMethod(
            channel,
            '/ont.rpc.protocol.ProtocolService/get_current_protocol_run',
            minknow.rpc.protocol_pb2.GetCurrentProtocolRunRequest,
            minknow.rpc.protocol_pb2.ProtocolRunInfo,
        )
        self.watch_current_protocol_run = grpclib.client.UnaryStreamMethod(
            channel,
            '/ont.rpc.protocol.ProtocolService/watch_current_protocol_run',
            minknow.rpc.protocol_pb2.WatchCurrentProtocolRunRequest,
            minknow.rpc.protocol_pb2.ProtocolRunInfo,
        )
        self.list_protocols = grpclib.client.UnaryUnaryMethod(
            channel,
            '/ont.rpc.protocol.ProtocolService/list_protocols',
            minknow.rpc.protocol_pb2.ListProtocolsRequest,
            minknow.rpc.protocol_pb2.ListProtocolsResponse,
        )
        self.get_context_info = grpclib.client.UnaryUnaryMethod(
            channel,
            '/ont.rpc.protocol.ProtocolService/get_context_info',
            minknow.rpc.protocol_pb2.GetContextInfoRequest,
            minknow.rpc.protocol_pb2.GetContextInfoResponse,
        )
        self.set_context_info = grpclib.client.UnaryUnaryMethod(
            channel,
            '/ont.rpc.protocol.ProtocolService/set_context_info',
            minknow.rpc.protocol_pb2.SetContextInfoRequest,
            minknow.rpc.protocol_pb2.SetContextInfoResponse,
        )
        self.get_sample_id = grpclib.client.UnaryUnaryMethod(
            channel,
            '/ont.rpc.protocol.ProtocolService/get_sample_id',
            minknow.rpc.protocol_pb2.GetSampleIdRequest,
            minknow.rpc.protocol_pb2.GetSampleIdResponse,
        )
        self.set_sample_id = grpclib.client.UnaryUnaryMethod(
            channel,
            '/ont.rpc.protocol.ProtocolService/set_sample_id',
            minknow.rpc.protocol_pb2.SetSampleIdRequest,
            minknow.rpc.protocol_pb2.SetSampleIdResponse,
        )
        self.get_protocol_purpose = grpclib.client.UnaryUnaryMethod(
            channel,
            '/ont.rpc.protocol.ProtocolService/get_protocol_purpose',
            minknow.rpc.protocol_pb2.GetProtocolPurposeRequest,
            minknow.rpc.protocol_pb2.GetProtocolPurposeResponse,
        )
        self.set_protocol_purpose = grpclib.client.UnaryUnaryMethod(
            channel,
            '/ont.rpc.protocol.ProtocolService/set_protocol_purpose',
            minknow.rpc.protocol_pb2.SetProtocolPurposeRequest,
            minknow.rpc.protocol_pb2.SetProtocolPurposeResponse,
        )
        self.add_epi2me_workflow = grpclib.client.UnaryUnaryMethod(
            channel,
            '/ont.rpc.protocol.ProtocolService/add_epi2me_workflow',
            minknow.rpc.protocol_pb2.AddEpi2meWorkflowRequest,
            minknow.rpc.protocol_pb2.AddEpi2meWorkflowResponse,
        )
        self.list_protocol_group_ids = grpclib.client.UnaryUnaryMethod(
            channel,
            '/ont.rpc.protocol.ProtocolService/list_protocol_group_ids',
            minknow.rpc.protocol_pb2.ListProtocolGroupIdsRequest,
            minknow.rpc.protocol_pb2.ListProtocolGroupIdsResponse,
        )



# Generated by the Protocol Buffers compiler. DO NOT EDIT!
# source: minknow/rpc/keystore.proto
# plugin: grpclib.plugin.main
import abc
import typing

import grpclib.const
import grpclib.client
if typing.TYPE_CHECKING:
    import grpclib.server

import google.protobuf.any_pb2
from . import rpc_options_pb2
from . import keystore_pb2


class KeyStoreServiceBase(abc.ABC):

    @abc.abstractmethod
    async def store(self, stream: 'grpclib.server.Stream[minknow.rpc.keystore_pb2.StoreRequest, minknow.rpc.keystore_pb2.StoreResponse]') -> None:
        pass

    @abc.abstractmethod
    async def remove(self, stream: 'grpclib.server.Stream[minknow.rpc.keystore_pb2.RemoveRequest, minknow.rpc.keystore_pb2.RemoveResponse]') -> None:
        pass

    @abc.abstractmethod
    async def get_one(self, stream: 'grpclib.server.Stream[minknow.rpc.keystore_pb2.GetOneRequest, minknow.rpc.keystore_pb2.GetOneResponse]') -> None:
        pass

    @abc.abstractmethod
    async def get(self, stream: 'grpclib.server.Stream[minknow.rpc.keystore_pb2.GetRequest, minknow.rpc.keystore_pb2.GetResponse]') -> None:
        pass

    @abc.abstractmethod
    async def watch(self, stream: 'grpclib.server.Stream[minknow.rpc.keystore_pb2.WatchRequest, minknow.rpc.keystore_pb2.WatchResponse]') -> None:
        pass

    def __mapping__(self) -> typing.Dict[str, grpclib.const.Handler]:
        return {
            '/ont.rpc.keystore.KeyStoreService/store': grpclib.const.Handler(
                self.store,
                grpclib.const.Cardinality.UNARY_UNARY,
                minknow.rpc.keystore_pb2.StoreRequest,
                minknow.rpc.keystore_pb2.StoreResponse,
            ),
            '/ont.rpc.keystore.KeyStoreService/remove': grpclib.const.Handler(
                self.remove,
                grpclib.const.Cardinality.UNARY_UNARY,
                minknow.rpc.keystore_pb2.RemoveRequest,
                minknow.rpc.keystore_pb2.RemoveResponse,
            ),
            '/ont.rpc.keystore.KeyStoreService/get_one': grpclib.const.Handler(
                self.get_one,
                grpclib.const.Cardinality.UNARY_UNARY,
                minknow.rpc.keystore_pb2.GetOneRequest,
                minknow.rpc.keystore_pb2.GetOneResponse,
            ),
            '/ont.rpc.keystore.KeyStoreService/get': grpclib.const.Handler(
                self.get,
                grpclib.const.Cardinality.UNARY_UNARY,
                minknow.rpc.keystore_pb2.GetRequest,
                minknow.rpc.keystore_pb2.GetResponse,
            ),
            '/ont.rpc.keystore.KeyStoreService/watch': grpclib.const.Handler(
                self.watch,
                grpclib.const.Cardinality.UNARY_STREAM,
                minknow.rpc.keystore_pb2.WatchRequest,
                minknow.rpc.keystore_pb2.WatchResponse,
            ),
        }


class KeyStoreServiceStub:

    def __init__(self, channel: grpclib.client.Channel) -> None:
        self.store = grpclib.client.UnaryUnaryMethod(
            channel,
            '/ont.rpc.keystore.KeyStoreService/store',
            minknow.rpc.keystore_pb2.StoreRequest,
            minknow.rpc.keystore_pb2.StoreResponse,
        )
        self.remove = grpclib.client.UnaryUnaryMethod(
            channel,
            '/ont.rpc.keystore.KeyStoreService/remove',
            minknow.rpc.keystore_pb2.RemoveRequest,
            minknow.rpc.keystore_pb2.RemoveResponse,
        )
        self.get_one = grpclib.client.UnaryUnaryMethod(
            channel,
            '/ont.rpc.keystore.KeyStoreService/get_one',
            minknow.rpc.keystore_pb2.GetOneRequest,
            minknow.rpc.keystore_pb2.GetOneResponse,
        )
        self.get = grpclib.client.UnaryUnaryMethod(
            channel,
            '/ont.rpc.keystore.KeyStoreService/get',
            minknow.rpc.keystore_pb2.GetRequest,
            minknow.rpc.keystore_pb2.GetResponse,
        )
        self.watch = grpclib.client.UnaryStreamMethod(
            channel,
            '/ont.rpc.keystore.KeyStoreService/watch',
            minknow.rpc.keystore_pb2.WatchRequest,
            minknow.rpc.keystore_pb2.WatchResponse,
        )



# -*- coding: utf-8 -*-

"""Console script for porerefiner."""
import sys, os
import click

from asyncio import run
from pathlib import Path
from functools import wraps

from porerefiner.cli_utils import VALID_RUN_ID, server, hr_formatter, json_formatter, xml_formatter, handle_connection_errors
from porerefiner.samplesheets import load_from_csv, load_from_excel
from porerefiner.protocols.porerefiner.rpc.porerefiner_pb2 import RunRequest, RunListRequest, RunAttachRequest, RunRsyncRequest, TagRequest

# default_config = lambda: os.environ.get('POREREFINER_CONFIG', Path.home() / '.porerefiner' / 'config.yaml')

def default_config():
    path = Path(os.environ.get('POREREFINER_CONFIG', Path.home() / '.porerefiner' / 'config.yaml'))
    if not path.exists():
        path = Path("/etc/porerefiner/config.yaml")
        if not path.exists():
            from porerefiner.config import Config
            socket_path = click.prompt('path to porerefiner server socket?', default=Path('/etc/porerefiner/porerefiner.sock'), show_default=True)
            Config.new_config_file(path, client_only=True, socket_path=socket_path)
    return path



with_config = click.option('--config', default=default_config(), help='Path to PoreRefiner config', show_default=True)

def coroutine(func):
    "Coroutine runner"
    @wraps(func)
    def wrapper(*a, **k):
        return run(func(*a, **k))
    return wrapper

@click.group()
def cli():
    """Command line interface for PoreRefiner, a Nanopore run manager."""
    pass #pragma: no cover


@cli.command()
@handle_connection_errors
@with_config
@click.option('-a', '--all', is_flag=True, default=False, help="Show finished and ongoing runs.")
@click.option('-h', '--human-readable', 'output_format', flag_value=hr_formatter, help='Output in a human-readable table.', default=hr_formatter)
@click.option('-j', '--json', 'output_format', flag_value=json_formatter, help='Output in JSON.')
@click.option('-x', '--xml', 'output_format', flag_value=xml_formatter, help='Output in schemaless XML.')
@click.option('-t', '--tag', 'tags', multiple=True)
@click.option('-e', '--extended', 'extend', default=False, is_flag=True, help="Extended output format.")
@coroutine
async def ps(config, output_format, extend, all=False, tags=[]):
    "Show runs in progress, or every tracked run (--all), or with a particular tag (--tag)."
    with server(config) as serv:
        with output_format(extend) as formatter:
            resp = await serv.GetRuns(RunListRequest(all=all, tags=tags))
            for run in resp.runs.runs:
                formatter(run)

@cli.command()
@handle_connection_errors
@click.option('-h', '--human-readable', 'output_format', flag_value=hr_formatter, help='Output in a human-readable table.', default=hr_formatter)
@click.option('-j', '--json', 'output_format', flag_value=json_formatter, help='Output in JSON.')
@click.option('-x', '--xml', 'output_format', flag_value=xml_formatter, help='Output in schemaless XML.')
@click.argument('run_id', type=VALID_RUN_ID)
@coroutine
async def info(output_format, run_id, config=default_config()):
    "Return information about a run, historical or in progress."
    with server(config) as serv:
        with output_format(extend=True) as formatter:
            req = RunRequest()
            if isinstance(run_id, str):
                req.name = run_id
            else:
                req.id = run_id
            resp = await serv.GetRunInfo(req)
            if resp.HasField('error'):
                click.echo(f"ERROR: {resp.error.err_message}", err=True)
                quit(1)
            else:
                formatter(resp.run)

@cli.command()
def template():
    "Write a sample sheet template to STDOUT."
    click.echo("""porerefiner_ver,1.0.1
library_id,
sequencing_kit,
barcode_kits,
sample_id,accession,barcode_id,organism,extraction_kit,comment,user
""")

async def tag_runner(config, run_id, tags=[], untag=False):
    with server(config) as serv:
        req = RunRequest()
        if isinstance(run_id, str):
            req.name = run_id
        else:
            req.id = run_id
        resp = await serv.GetRunInfo(req)
        if resp.HasField('error'):
            click.echo(f"ERROR: {resp.error.err_message}", err=True)
            quit(1)
        else:
            run_id = resp.run.id
            return await serv.Tag(TagRequest(id=run_id, tags=tags, untag=untag))

@cli.command()
@handle_connection_errors
@with_config
@click.argument('run_id', type=VALID_RUN_ID)
@click.argument('tag', type=click.STRING, nargs=-1)
def tag(config, run_id, tag=[]):
    "Add one or more tags to a run."
    run(tag_runner(config, run_id, tag))


@cli.command()
@handle_connection_errors
@with_config
@click.argument('run_id', type=VALID_RUN_ID)
@click.argument('tag', type=click.STRING, nargs=-1)
def untag(config, run_id, tag=[]):
    "Remove one or more tags from a run."
    run(tag_runner(config, run_id, tag, untag=True))

@cli.command()
@handle_connection_errors
@with_config
@click.argument('samplesheet', type=click.File('rb'))
@click.option('-r', '--run', 'run_id', type=VALID_RUN_ID, )
@coroutine
async def load(config, samplesheet, run_id=None):
    "Load a sample sheet to be attached to a run, or to the next run that is started."
    try:
        if 'csv' in samplesheet.name:
            ss = load_from_csv(samplesheet)
        elif 'tsv' in samplesheet.name or 'txt' in samplesheet.name:
            ss = load_from_csv(samplesheet, delimiter='\t')
        elif 'xls' in samplesheet.name:
            ss = load_from_excel(samplesheet)
    except TypeError as e:
        click.echo(e, err=True)
    except ImportError:
        click.echo(f"ERROR: OpenPyXL not installed; Excel files ({samplesheet.name}) can't be read. Use pip to install OpenPyXL.", err=True)
    else:
        with server(config) as serv:
            req = RunAttachRequest(sheet=ss)
            if isinstance(run_id, int):
                req.id = run_id
            elif run_id:
                req.name = run_id
            resp = await serv.AttachSheetToRun(req)
            if resp.error:
                click.echo(resp.error.err_message, err=True)
                quit(1)



if __name__ == "__main__":
    sys.exit(cli())  # pragma: no cover


from abc import ABCMeta, abstractmethod
from typing import Union

from tempfile import mkdtemp
from pathlib import Path

import logging
import pkgutil

from porerefiner.cli_utils import render_dataclass, Email, Url, PathStr

SUBMITTERS = [] # configured (reified) submitters

REGISTRY = {} # available submitter classes

log = logging.getLogger('porerefiner.submitter.registry')

class _MetaRegistry(type):

    def __new__(meta, name, bases, class_dict):
        cls = type.__new__(meta, name, bases, class_dict)
        if cls not in REGISTRY:
            REGISTRY[name] = cls
        return cls

    def __call__(cls, *args, **kwargs):
        the_instance = super().__call__(*args, **kwargs)
        SUBMITTERS.append(the_instance)
        log.debug(cls.__name__)
        return the_instance

class RegisteringABCMeta(ABCMeta, _MetaRegistry):
    pass

class Submitter(metaclass=RegisteringABCMeta):

    def __repr__(self):
        return type(self).__name__

    @abstractmethod
    async def test_noop(self) -> None:
        "No-op method submitters should implement to make sure the submitter can access an external resource."
        pass

    @abstractmethod
    def reroot_path(self, path) -> Path:
        "Submitters should translate paths to execution environment"
        pass

    async def _submit(self, job):
        "Create datadir, delegate job setup, then call subclass method to submit job"
        from porerefiner.jobs import FileJob, RunJob, CONFIGURED_JOB_REGISTRY
        run = job.run
        file = job.file
        logg = log.getChild(type(self).__name__)
        hints = {}
        datadir = job.datadir = Path(mkdtemp())
        remotedir = job.remotedir = self.reroot_path(datadir)
        job.save()
        configured_job = CONFIGURED_JOB_REGISTRY[job.job_class]
        if isinstance(configured_job, RunJob):
            cmd = configured_job.setup(run=run, datadir=datadir, remotedir=remotedir)
        elif isinstance(configured_job, FileJob):
            cmd = configured_job.setup(file=file, run=file.run, datadir=datadir, remotedir=remotedir)
        if isinstance(cmd, tuple): #some jobs return a string plus execution hints
            cmd, hints = cmd
        if not isinstance(cmd, str): #has to be a string
            raise RuntimeError("job setup method needs to return a string, or a string and dictionary")
        cmd = " ".join(cmd.split()) # turn tabs and returns into spaces
        logg.getChild('cmd').debug(cmd)
        try:
            job.job_id = await self.begin_job(cmd, datadir, remotedir, environment_hints=hints)
            job.status = 'QUEUED'
        except Exception as e:
            logg.error(e)
            job.job_state.attempts += 1
            if job.job_state.attempts > 3:
                job.status = 'FAILED'
                raise
        finally:
            job.save()



    @abstractmethod
    async def begin_job(self, execution_string, datadir, remotedir, environment_hints={}) -> str:
        "Semantics of scheduling a job"
        pass

    async def _poll(self, job):
        logg = log.getChild(type(self).__name__)
        try:
            job.status = status = await self.poll_job(job)
        except Exception as e:
            job.status = 'FAILED'
            raise
        finally:
            job.save()


    @abstractmethod
    async def poll_job(self, job) -> str:
        pass

    def _close(self, job):
        logg = log.getChild(type(self).__name__)
        return self.closeout_job(job, job.datadir, job.remotedir)

    @abstractmethod
    def closeout_job(self, job, datadir, remotedir) -> None:
        pass

    @classmethod
    def get_configurable_options(cls):
        "Enumerate configurable options and value type as a guide to configuration."
        return render_dataclass(cls)




import asyncio
import aiohttp
import click
import datetime
import json
import logging
import sys
import watchdog

from asyncio import run, gather, wait
from datetime import datetime, timedelta
from grpclib.server import Server
from grpclib.utils import graceful_exit
from hachiko.hachiko import AIOEventHandler, AIOWatchdog
from itertools import chain
from peewee import JOIN
from porerefiner import models
from porerefiner.models import Run, Qa, File, Job, SampleSheet, Sample, Tag, TagJunction
from porerefiner.cli_utils import relativize_path as r, absolutize_path as a, json_formatter
from porerefiner.jobs import poll_jobs, CLASS_REGISTRY, JOBS
from os.path import split, getmtime
from os import remove
from pathlib import Path

from porerefiner.protocols.minknow.rpc.manager_grpc import ManagerServiceStub
from porerefiner.protocols.porerefiner.rpc.porerefiner_pb2 import Run as RunMessage, RunList, RunResponse, Error, GenericResponse, RunRsyncResponse, RunListResponse
from porerefiner.protocols.porerefiner.rpc.porerefiner_grpc import PoreRefinerBase
from porerefiner.notifiers import NOTIFIERS



log = logging.getLogger('porerefiner.rpc')


def get_run(run_id):
    run = Run.get_or_none(Run.pk == run_id) or Run.get_or_none(Run.name == run_id) or Run.get_or_none(Run.alt_name == run_id)
    if not run:
        raise ValueError(f"Run id or name '{run_id}' not found.")
    return run

def make_run_msg(run):
    return RunMessage(id=run.pk,
        name=run.name,
        mnemonic_name=run.alt_name,
        library_id=run.library_id,
        status=run.status,
        path=a(run.path),
        # flowcell_type=run.flowcell.consumable_type,
        # flowcell_id=run.flowcell.consumable_id,
        basecalling_model=run.basecalling_model,
        sequencing_kit=run.sample_sheet.barcoding_kit,
        samples=[
            RunMessage.Sample(id=sample.pk,
                    name=sample.sample_id,
                    accession=sample.accession,
                    barcode_id=sample.barcode_id,
                    barcode_seq=sample.barcode_seq,
                    organism=sample.organism,
                    extraction_kit=sample.extraction_kit,
                    comment=sample.comment,
                    user=sample.user,
                    files=[
                    RunMessage.File(name=file.name,
                            path=a(file.path),
                            size=file.path.stat().st_size,
                            ready=datetime.now() - file.last_modified > timedelta(hours=1),
                            hash=file.checksum,
                            tags=[tag.name for tag in file.tags])
                    for file in sample.files],
                    tags=[tag.name for tag in sample.tags]
                    ) for sample in run.samples
        ],
        files=[
            RunMessage.File(name=file.name,
                path=a(file.path),
                size=file.path.stat().st_size,
                ready=datetime.now() - file.last_modified > timedelta(hours=1),
                hash=file.checksum,
                tags=[tag.name for tag in file.tags])
        for file in run.files],
        tags=[tag.name for tag in run.tags]
        )

async def get_run_info(run_id):
    return make_run_msg(get_run(run_id))

async def list_runs(all=False, tags=[]):
    if tags:
        #implies all
        return [make_run_msg(run) for run in Run.select().join(TagJunction).join(Tag).where(Tag.name << tags)]
    if all:
        return [make_run_msg(run) for run in Run.select()]
    #only in-progress runs
    return [make_run_msg(run) for run in Run.select().where(Run.ended.is_null())]




class PoreRefinerDispatchServer(PoreRefinerBase):
    "Eventhandler for RPC events coming from command line or Flask app"

    async def GetRuns(self, stream: 'grpclib.server.Stream[porerefiner_pb2.RunListRequest, porerefiner_pb2.RunList]') -> None:
        log.debug("API call: Get Runs")
        request = await stream.recv_message()
        log.debug(f"all:{request.all}, tags:{request.tags}")
        await stream.send_message(RunListResponse(runs=RunList(runs = await list_runs(request.all, request.tags))))
        log.debug("Response sent")

    async def GetRunInfo(self, stream: 'grpclib.server.Stream[porerefiner_pb2.RunRequest, porerefiner_pb2.RunResponse]') -> None:
        request = await stream.recv_message()
        log.debug(f"API call: get run info for run {request.id or request.name}")
        try:
            run_msg = await get_run_info(request.id or request.name)
            reply_msg = RunResponse(run=run_msg)
        except ValueError as e:
            err_msg = Error(type='ValueError', err_message=str(e))
            reply_msg = RunResponse(error=err_msg)
        await stream.send_message(reply_msg)
        log.debug("Response sent")

    async def AttachSheetToRun(self, stream: 'grpclib.server.Stream[porerefiner_pb2.RunAttachRequest, porerefiner_pb2.RunAttachResponse]') -> None:
        request = await stream.recv_message()
        log.debug("API call: Attach sample sheet")
        run = None
        try:
            run = get_run(request.id or request.name)
        except ValueError: #no run
            query = Run.get_unannotated_runs()
            if query.count() == 1:
                run = next(query)
        ss = SampleSheet.new_sheet_from_message(request.sheet, run)
        await stream.send_message(GenericResponse())
        log.debug("Response sent")

    async def RsyncRunTo(self, stream: 'grpclib.server.Stream[porerefiner_pb2.RunRsyncRequest, porerefiner_pb2.RunRsyncResponse]') -> None:
        request = await stream.recv_message()
        log.debug(f"API call: send run via rsync to {request.dest}")
        await stream.send_message(RunRsyncResponse())
        log.debug("Response sent")

    async def Tag(self, stream):
        request = await stream.recv_message()
        log.debug(f"API call: tag run {request.id} with tags '{request.tags}'")
        run = Run.get_or_none(pk=request.id)
        resp = GenericResponse()
        if run:
            for tag in request.tags:
                if request.untag:
                    run.untag(tag)
                else:
                    run.tag(tag)
        else:
            resp.error = Error(type="NoSuchRun", err_message=f"run id {request.id} not found.")
        await stream.send_message(resp)
        log.info("Response sent")

async def start_server(socket, *a, **k):
    "Coroutine to bring up the rpc server"
    server = Server([PoreRefinerDispatchServer()])
    with graceful_exit([server]):
        await server.start(path=str(socket))
        log.info(f"RPC server listening on {socket}...")
        await server.wait_closed()
        log.warning(f"RPC server shutting down.")






from dataclasses import dataclass, field
from os import environ

from porerefiner.jobs import FileJob, RunJob

@dataclass
class GenericFileJob(FileJob):

    command: str
    hints: dict = field(default_factory=dict)

    def setup(self, run, file, datadir, remotedir):
        locals().update(environ)
        return self.command.format(**locals())

    def collect(*args, **kwargs):
        pass

@dataclass
class GenericRunJob(RunJob):

    command: str
    hints: dict = field(default_factory=dict)

    def setup(self, run, datadir, remotedir):
        locals().update(environ)
        return self.command.format(**locals())

    def collect(*args, **kwargs):
        pass




from unittest import TestCase, skip
from unittest.mock import Mock, patch

from tests import paths, with_database, TestBase, sql_ints, Model, jobs as _jobs

from porerefiner import models, jobs

from hypothesis import given, strategies as strat, example, seed, settings, HealthCheck
#from hypothesis_fspaths import fspaths, _PathLike

from datetime import datetime
import pathlib
import sys



# safe_paths = lambda: fspaths().filter(lambda x: isinstance(x, str) or isinstance(x, _PathLike))
class TestJobDefinition(jobs.AbstractJob):
    pass


class TestModels(TestCase):

    @given(paths())
    @example(b'/path/pa')
    def test_path_field(self, path):
        try:
            pa = pathlib.Path(path)
        except TypeError:
            pa = pathlib.Path(str(path, encoding=sys.getfilesystemencoding()))
        fld = models.PathField()
        self.assertEqual(fld.python_value(fld.db_value(path)), pa)

    # @given(job=_jobs())
    # def test_job_field(self, job):
    #     fld = models.JobField()
    #     self.assertEqual(type(fld.python_value(fld.db_value(job))), type(job))

    def test_models_registered(self):
        self.assertEqual(len(models.REGISTRY), 9)

    # @skip('broken')
    @given(tag=strat.text().filter(lambda x: x))
    @with_database
    def test_tags(self, tag):
        flow = models.SampleSheet.create()
        tag, _ = models.Tag.get_or_create(name=tag)
        tag_j = models.TagJunction.create(samplesheet=flow, tag=tag)
        self.assertIn(tag, flow.tags)

    @with_database
    def test_tag_failure(self):
        with self.assertRaises(Exception):
            tag = models.Tag.create(name='')

# class TestFlowcell(TestCase):

#     @given(pk=sql_ints(),
#            consumable_id=strat.text(),
#            consumable_type=strat.text(),
#            path=paths())
#     @with_database
#     def test_flowcell(self, **kwargs):
#         assert models.Flowcell.create(**kwargs)

class TestRun(TestCase):

    @skip('broken')
    @given(pk=sql_ints(),
           name=strat.text(),
           library_id=strat.text(),
           alt_name=strat.text(),
           run_id=strat.text(),
           started=strat.datetimes().filter(lambda d: d < datetime.now()),
           ended=strat.datetimes().filter(lambda d: d > datetime.now()),
           path=paths(),
           basecalling_model=strat.one_of(*[strat.just(val) for val, _ in models.Run.basecallers]))
    @with_database
    def test_run(self, **kwargs):
        self.flow = models.Flowcell.create(consumable_id='TEST',
                                           consumable_type='TEST',
                                           path='TEST/TEST')
        assert models.Run.create(flowcell=self.flow, **kwargs).run_duration

    @settings(deadline=None, suppress_health_check=(HealthCheck.all()))
    @given(run=Model.Runs(),
           job=_jobs())
    @with_database
    def test_job_spawn(self, run, job):
        # run.flowcell.save()
        run.save()
        self.assertIsNotNone(run.pk)
        jobb = run.spawn(job)
        self.assertIs(job, jobb.job_state)

class TestQa(TestCase):

    @given(pk=sql_ints(),
           coverage=strat.floats().filter(lambda f: f > 0),
           quality=strat.floats().filter(lambda f: f > 0))
    @with_database
    def test_qa(self, **kwargs):
        assert models.Qa.create(**kwargs)



class TestJob(TestCase):

    @given(job=Model.Jobs())
    @with_database
    def test_job(self, job):
        assert job.save()

    # @skip('no test yet')
    @given(job=Model.Jobs(),
           path=paths(pathlib_only=True))
    @with_database
    def test_job_files(self, job, path):
        job.save()
        # file = models.File(path=path)
        # file.save()
        file = models.File.create(path=path)
        job.files.add(file)
        job.save()
        self.assertIn(file, job.files)

class TestSampleSheet(TestCase):

    @given(pk=sql_ints(),
           path=paths(),
           date=strat.datetimes(),
           sequencing_kit=strat.text())
    @with_database
    def test_samplesheet(self, **kwargs):
        assert models.SampleSheet.create(**kwargs)

    @skip('broken')
    @with_database
    def test_get_unused_sheets(self):
        self.flow = flow = models.Flowcell.create(consumable_id="TEST|TEST|TEST", consumable_type="TEST|TEST|TEST", path="TEST/TEST/TEST")
        self.run = models.Run.create(pk=100, library_id='x', name="TEST", flowcell=flow, path="TEST/TEST/TEST")
        self.assertFalse(models.SampleSheet.get_unused_sheets().count())
        models.SampleSheet.create(path="TEST")
        self.assertEqual(models.SampleSheet.get_unused_sheets().count(), 1)

    @skip('broken')
    @seed(5249283748837843916514315999694345497)
    @given(ss=Model.Samplesheets())
    @with_database
    def test_new_sheet_from_message(self, ss):
        flow = models.Flowcell.create(consumable_id="TEST|TEST|TEST", consumable_type="TEST|TEST|TEST", path="TEST/TEST/TEST")
        run = models.Run.create(pk=100, library_id='x', name="TEST", flowcell=flow, path="TEST/TEST/TEST")
        s = models.SampleSheet.new_sheet_from_message(ss, run)
        self.assertEqual(run.sample_sheet, s)

class TestSample(TestCase):

    @given(pk=sql_ints(),
           sample_id=strat.text(),
           accession=strat.text(),
           barcode_id=strat.text(),
           organism=strat.text(),
           extraction_kit=strat.text(),
           comment=strat.text(),
           user=strat.emails())
    @with_database
    def test_sample(self, **k):
        ss = models.SampleSheet.create(path=k['sample_id'])
        assert models.Sample.create(samplesheet=ss, **k)

class TestFile(TestCase):

    @given(pk=sql_ints(),
           path=paths(),
           checksum=strat.text(),
           last_modified=strat.datetimes(),
           exported=strat.booleans())
    @with_database
    def test_file(self, **k):
        assert models.File.create(**k)

    @given(pk=sql_ints(),
           path=paths(),
           checksum=strat.text(),
           last_modified=strat.datetimes(),
           exported=strat.booleans(),
           job=Model.Jobs())
    @with_database
    def test_job_spawn(self, job, **k):
        fi = models.File.create(**k)
        assert fi.spawn(job)


    
